@@ -18,7 +18,7 @@
  * Split out from ipsec_init.c version 1.70.
  */
 
-char ipsec_proc_c_version[] = "RCSID $Id: ipsec_proc.c,v 1.24 2003/06/20 01:42:21 mcr Exp $";
+char ipsec_proc_c_version[] = "RCSID $Id: ipsec_proc.c,v 1.36 2004/04/12 19:48:31 dhr Exp $";
 
 #include <linux/config.h>
 #include <linux/version.h>
@@ -73,18 +73,14 @@ char ipsec_proc_c_version[] = "RCSID $Id: ipsec_proc.c,v 1.24 2003/06/20 01:42:2
 
 #include "freeswan/ipsec_encap.h"
 #include "freeswan/ipsec_radij.h"
-#include "freeswan/ipsec_netlink.h"
 #include "freeswan/ipsec_xform.h"
 #include "freeswan/ipsec_tunnel.h"
 #include "freeswan/ipsec_xmit.h"
 
 #include "freeswan/ipsec_rcv.h"
-#include "freeswan/ipsec_ah.h"
-#include "freeswan/ipsec_esp.h"
 
-#ifdef CONFIG_IPSEC_IPCOMP
-#include "freeswan/ipcomp.h"
-#endif /* CONFIG_IPSEC_IPCOMP */
+#include "freeswan/ipsec_esp.h"
+#include "freeswan/ipsec_ipcomp.h"
 
 #include "freeswan/ipsec_proto.h"
 
@@ -105,6 +101,31 @@ static struct proc_dir_entry *proc_stats_dir     = NULL;
 struct ipsec_birth_reply ipsec_ipv4_birth_packet;
 struct ipsec_birth_reply ipsec_ipv6_birth_packet;
 
+/* ipsec_snprintf: like snprintf except
+ * - size is signed and a negative value is treated as if it were 0
+ * - the returned result is never negative --
+ *   an error generates a "?" or null output (depending on space).
+ *   (Our callers are too lazy to check for an error return.)
+ */
+int ipsec_snprintf(char *buf, ssize_t size, const char *fmt, ...)
+{
+	va_list args;
+	int i;
+	size_t possize = size < 0? 0 : size;
+
+	va_start(args, fmt);
+	i = vsnprintf(buf,possize,fmt,args);
+	va_end(args);
+	if (i < 0) {
+	    /* create empty output in place of error */
+	    i = 0;
+	    if (size > 0) {
+		*buf = '\0';
+	    }
+	}
+	return i;
+}
+
 IPSEC_PROCFS_DEBUG_NO_STATIC
 int
 ipsec_eroute_get_info(char *buffer, 
@@ -112,12 +133,12 @@ ipsec_eroute_get_info(char *buffer,
 		      off_t offset, 
 		      int length        IPSEC_PROC_LAST_ARG)
 {
-	struct wsbuf w = {buffer, length, offset, 0, 0, 0, 0};
+	struct wsbuf w = {buffer, length, offset, 0, 0};
 
-#ifdef CONFIG_IPSEC_DEBUG
+#ifdef CONFIG_KLIPS_DEBUG
 	if (debug_radij & DB_RJ_DUMPTREES)
 	  rj_dumptrees();			/* XXXXXXXXX */
-#endif /* CONFIG_IPSEC_DEBUG */
+#endif /* CONFIG_KLIPS_DEBUG */
 
 	KLIPS_PRINT(debug_tunnel & DB_TN_PROCFS,
 		    "klips_debug:ipsec_eroute_get_info: "
@@ -135,10 +156,7 @@ ipsec_eroute_get_info(char *buffer,
 	spin_unlock_bh(&eroute_lock);
 
 	*start = buffer + (offset - w.begin);	/* Start of wanted data */
-	w.len -= (offset - w.begin);			/* Start slop */
-	if (w.len > length)
-		w.len = length;
-	return w.len;
+	return w.len - (offset - w.begin);
 }
 
 IPSEC_PROCFS_DEBUG_NO_STATIC
@@ -148,11 +166,12 @@ ipsec_spi_get_info(char *buffer,
 		   off_t offset,
 		   int length    IPSEC_PROC_LAST_ARG)
 {
+	const int max_content = length > 0? length-1 : 0;	/* limit of useful snprintf output */
 	int len = 0;
-	off_t pos = 0, begin = 0;
+	off_t begin = 0;
 	int i;
 	struct ipsec_sa *sa_p;
-	char sa[SATOA_BUF];
+	char sa[SATOT_BUF];
 	char buf_s[SUBNETTOA_BUF];
 	char buf_d[SUBNETTOA_BUF];
 	size_t sa_len;
@@ -167,28 +186,26 @@ ipsec_spi_get_info(char *buffer,
 	
 	spin_lock_bh(&tdb_lock);
 
-
-	
 	for (i = 0; i < SADB_HASHMOD; i++) {
 		for (sa_p = ipsec_sadb_hash[i];
 		     sa_p;
 		     sa_p = sa_p->ips_hnext) {
 			atomic_inc(&sa_p->ips_refcount);
-			sa_len = satoa(sa_p->ips_said, 0, sa, SATOA_BUF);
-			len += sprintf(buffer + len, "%s ",
+			sa_len = satot(&sa_p->ips_said, 'x', sa, sizeof(sa));
+			len += ipsec_snprintf(buffer+len, length-len, "%s ",
 				       sa_len ? sa : " (error)");
 
-			len += sprintf(buffer + len, "%s%s%s",
+			len += ipsec_snprintf(buffer+len, length-len, "%s%s%s",
 				       IPS_XFORM_NAME(sa_p));
 
-			len += sprintf(buffer + len, ": dir=%s",
+			len += ipsec_snprintf(buffer+len, length-len, ": dir=%s",
 				       (sa_p->ips_flags & EMT_INBOUND) ?
 				       "in " : "out");
 
 			if(sa_p->ips_addr_s) {
 				addrtoa(((struct sockaddr_in*)(sa_p->ips_addr_s))->sin_addr,
 					0, buf_s, sizeof(buf_s));
-				len += sprintf(buffer + len, " src=%s",
+				len += ipsec_snprintf(buffer+len, length-len, " src=%s",
 					       buf_s);
 			}
 
@@ -206,82 +223,82 @@ ipsec_spi_get_info(char *buffer,
 					  buf_d,
 					  sizeof(buf_d));
 
-				len += sprintf(buffer + len, " policy=%s->%s",
+				len += ipsec_snprintf(buffer+len, length-len, " policy=%s->%s",
 					       buf_s, buf_d);
 			}
 			
 			if(sa_p->ips_iv_bits) {
 				int j;
-				len += sprintf(buffer + len, " iv_bits=%dbits iv=0x",
+				len += ipsec_snprintf(buffer+len, length-len, " iv_bits=%dbits iv=0x",
 					       sa_p->ips_iv_bits);
 
 				for(j = 0; j < sa_p->ips_iv_bits / 8; j++) {
-					len += sprintf(buffer + len, "%02x",
+					len += ipsec_snprintf(buffer+len, length-len, "%02x",
 						       (__u32)((__u8*)(sa_p->ips_iv))[j]);
 				}
 			}
 
 			if(sa_p->ips_encalg || sa_p->ips_authalg) {
 				if(sa_p->ips_replaywin) {
-					len += sprintf(buffer + len, " ooowin=%d",
+					len += ipsec_snprintf(buffer+len, length-len, " ooowin=%d",
 						       sa_p->ips_replaywin);
 				}
 				if(sa_p->ips_errs.ips_replaywin_errs) {
-					len += sprintf(buffer + len, " ooo_errs=%d",
+					len += ipsec_snprintf(buffer+len, length-len, " ooo_errs=%d",
 						       sa_p->ips_errs.ips_replaywin_errs);
 				}
 				if(sa_p->ips_replaywin_lastseq) {
-                                       len += sprintf(buffer + len, " seq=%d",
+                                       len += ipsec_snprintf(buffer+len, length-len, " seq=%d",
 						      sa_p->ips_replaywin_lastseq);
 				}
 				if(sa_p->ips_replaywin_bitmap) {
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2,3,0)
-					len += sprintf(buffer + len, " bit=0x%Lx",
+					len += ipsec_snprintf(buffer+len, length-len, " bit=0x%Lx",
 						       sa_p->ips_replaywin_bitmap);
 #else
-					len += sprintf(buffer + len, " bit=0x%x%08x",
+					len += ipsec_snprintf(buffer+len, length-len, " bit=0x%x%08x",
 						       (__u32)(sa_p->ips_replaywin_bitmap >> 32),
 						       (__u32)sa_p->ips_replaywin_bitmap);
 #endif
 				}
 				if(sa_p->ips_replaywin_maxdiff) {
-					len += sprintf(buffer + len, " max_seq_diff=%d",
+					len += ipsec_snprintf(buffer+len, length-len, " max_seq_diff=%d",
 						       sa_p->ips_replaywin_maxdiff);
 				}
 			}
 			if(sa_p->ips_flags & ~EMT_INBOUND) {
-				len += sprintf(buffer + len, " flags=0x%x",
+				len += ipsec_snprintf(buffer+len, length-len, " flags=0x%x",
 					       sa_p->ips_flags & ~EMT_INBOUND);
-				len += sprintf(buffer + len, "<");
+				len += ipsec_snprintf(buffer+len, length-len, "<");
 				/* flag printing goes here */
-				len += sprintf(buffer + len, ">");
+				len += ipsec_snprintf(buffer+len, length-len, ">");
 			}
 			if(sa_p->ips_auth_bits) {
-				len += sprintf(buffer + len, " alen=%d",
+				len += ipsec_snprintf(buffer+len, length-len, " alen=%d",
 					       sa_p->ips_auth_bits);
 			}
 			if(sa_p->ips_key_bits_a) {
-				len += sprintf(buffer + len, " aklen=%d",
+				len += ipsec_snprintf(buffer+len, length-len, " aklen=%d",
 					       sa_p->ips_key_bits_a);
 			}
 			if(sa_p->ips_errs.ips_auth_errs) {
-				len += sprintf(buffer + len, " auth_errs=%d",
+				len += ipsec_snprintf(buffer+len, length-len, " auth_errs=%d",
 					       sa_p->ips_errs.ips_auth_errs);
 			}
 			if(sa_p->ips_key_bits_e) {
-				len += sprintf(buffer + len, " eklen=%d",
+				len += ipsec_snprintf(buffer+len, length-len, " eklen=%d",
 					       sa_p->ips_key_bits_e);
 			}
 			if(sa_p->ips_errs.ips_encsize_errs) {
-				len += sprintf(buffer + len, " encr_size_errs=%d",
+				len += ipsec_snprintf(buffer+len, length-len, " encr_size_errs=%d",
 					       sa_p->ips_errs.ips_encsize_errs);
 			}
 			if(sa_p->ips_errs.ips_encpad_errs) {
-				len += sprintf(buffer + len, " encr_pad_errs=%d",
+				len += ipsec_snprintf(buffer+len, length-len, " encr_pad_errs=%d",
 					       sa_p->ips_errs.ips_encpad_errs);
 			}
 			
-			len += sprintf(buffer + len, " life(c,s,h)=");
+			len += ipsec_snprintf(buffer+len, length-len, " life(c,s,h)=");
 
 			len += ipsec_lifetime_format(buffer + len,
 						     length - len,
@@ -315,55 +332,62 @@ ipsec_spi_get_info(char *buffer,
 			
 			if(sa_p->ips_life.ipl_usetime.ipl_last) { /* XXX-MCR should be last? */
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2,3,0)
-				len += sprintf(buffer + len, " idle=%Ld",
+				len += ipsec_snprintf(buffer+len, length-len, " idle=%Ld",
 					       jiffies / HZ - sa_p->ips_life.ipl_usetime.ipl_last);
 #else
-				len += sprintf(buffer + len, " idle=%lu",
+				len += ipsec_snprintf(buffer+len, length-len, " idle=%lu",
 					       jiffies / HZ - (unsigned long)sa_p->ips_life.ipl_usetime.ipl_last);
 #endif
 			}
 
-#ifdef CONFIG_IPSEC_IPCOMP
+#ifdef CONFIG_KLIPS_IPCOMP
 			if(sa_p->ips_said.proto == IPPROTO_COMP &&
 			   (sa_p->ips_comp_ratio_dbytes ||
 			    sa_p->ips_comp_ratio_cbytes)) {
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2,3,0)
-				len += sprintf(buffer + len, " ratio=%Ld:%Ld",
+ 				len += ipsec_snprintf(buffer+len, length-len, " ratio=%Ld:%Ld",
 					       sa_p->ips_comp_ratio_dbytes,
 					       sa_p->ips_comp_ratio_cbytes);
 #else
-				len += sprintf(buffer + len, " ratio=%lu:%lu",
+ 				len += ipsec_snprintf(buffer+len, length-len, " ratio=%lu:%lu",
 					       (unsigned long)sa_p->ips_comp_ratio_dbytes,
 					       (unsigned long)sa_p->ips_comp_ratio_cbytes);
 #endif
 			}
-#endif /* CONFIG_IPSEC_IPCOMP */
+#endif /* CONFIG_KLIPS_IPCOMP */
 
-			len += sprintf(buffer + len, " refcount=%d",
+ 			len += ipsec_snprintf(buffer+len, length-len, " refcount=%d",
 				       atomic_read(&sa_p->ips_refcount));
 
-			len += sprintf(buffer + len, " ref=%d",
+ 			len += ipsec_snprintf(buffer+len, length-len, " ref=%d",
 				       sa_p->ips_ref);
-#ifdef CONFIG_IPSEC_DEBUG
+#ifdef CONFIG_KLIPS_DEBUG
 			if(debug_xform) {
-			len += sprintf(buffer + len, " reftable=%lu refentry=%lu",
+ 			len += ipsec_snprintf(buffer+len, length-len, " reftable=%lu refentry=%lu",
 				       (unsigned long)IPsecSAref2table(sa_p->ips_ref),
 				       (unsigned long)IPsecSAref2entry(sa_p->ips_ref));
 			}
-#endif /* CONFIG_IPSEC_DEBUG */
+#endif /* CONFIG_KLIPS_DEBUG */
 
-			len += sprintf(buffer + len, "\n");
+ 			len += ipsec_snprintf(buffer+len, length-len, "\n");
 
-			pos = begin + len;
-			if(pos < offset) {
-				len = 0;
-				begin = pos;
-			}
-			if (pos > offset + length) {
-				atomic_dec(&sa_p->ips_refcount);
-				goto done_spi_i;
-			}
 			atomic_dec(&sa_p->ips_refcount);
+
+			if (len >= max_content) {
+				/* we've done all that can fit -- stop loops */
+				len = max_content;	/* truncate crap */
+				goto done_spi_i;
+			} else {
+				const off_t pos = begin + len;	/* file position of end of what we've generated */
+
+				if (pos <= offset) {
+					/* all is before first interesting character:
+					 * discard, but note where we are.
+					 */
+					len = 0;
+					begin = pos;
+				}
+			}
 		}
 	}
 
@@ -371,10 +395,7 @@ ipsec_spi_get_info(char *buffer,
 	spin_unlock_bh(&tdb_lock);
 
 	*start = buffer + (offset - begin);	/* Start of wanted data */
-	len -= (offset - begin);			/* Start slop */
-	if (len > length)
-		len = length;
-	return len;
+	return len - (offset - begin);
 }
 
 IPSEC_PROCFS_DEBUG_NO_STATIC
@@ -384,11 +405,12 @@ ipsec_spigrp_get_info(char *buffer,
 		      off_t offset,
 		      int length     IPSEC_PROC_LAST_ARG)
 {
+	const int max_content = length > 0? length-1 : 0;	/* limit of useful snprintf output */
 	int len = 0;
-	off_t pos = 0, begin = 0;
+	off_t begin = 0;
 	int i;
 	struct ipsec_sa *sa_p, *sa_p2;
-	char sa[SATOA_BUF];
+	char sa[SATOT_BUF];
 	size_t sa_len;
 
 	KLIPS_PRINT(debug_tunnel & DB_TN_PROCFS,
@@ -411,26 +433,33 @@ ipsec_spigrp_get_info(char *buffer,
 				sa_p2 = sa_p;
 				while(sa_p2 != NULL) {
 					atomic_inc(&sa_p2->ips_refcount);
-					sa_len = satoa(sa_p2->ips_said,
-						       0, sa, SATOA_BUF);
+					sa_len = satot(&sa_p2->ips_said,
+						       'x', sa, sizeof(sa));
 					
-					len += sprintf(buffer + len, "%s ",
+					len += ipsec_snprintf(buffer+len, length-len, "%s ",
 						       sa_len ? sa : " (error)");
 					atomic_dec(&sa_p2->ips_refcount);
 					sa_p2 = sa_p2->ips_onext;
 				}
-				len += sprintf(buffer + len, "\n");
-				pos = begin + len;
-				if(pos < offset) {
+				len += ipsec_snprintf(buffer+len, length-len, "\n");
+			}
+			atomic_dec(&sa_p->ips_refcount);
+
+			if (len >= max_content) {
+				/* we've done all that can fit -- stop loops */
+				len = max_content;	/* truncate crap */
+				goto done_spigrp_i;
+			} else {
+				const off_t pos = begin + len;	/* file position of end of what we've generated */
+
+				if (pos <= offset) {
+					/* all is before first interesting character:
+					 * discard, but note where we are.
+					 */
 					len = 0;
 					begin = pos;
 				}
-				if (pos > offset + length) {
-					atomic_dec(&sa_p->ips_refcount);
-					goto done_spigrp_i;
-				}
 			}
-			atomic_dec(&sa_p->ips_refcount);
 		}
 	}
 
@@ -438,12 +467,10 @@ ipsec_spigrp_get_info(char *buffer,
 	spin_unlock_bh(&tdb_lock);
 
 	*start = buffer + (offset - begin);	/* Start of wanted data */
-	len -= (offset - begin);			/* Start slop */
-	if (len > length)
-		len = length;
-	return len;
+	return len - (offset - begin);
 }
 
+
 IPSEC_PROCFS_DEBUG_NO_STATIC
 int
 ipsec_tncfg_get_info(char *buffer,
@@ -451,11 +478,12 @@ ipsec_tncfg_get_info(char *buffer,
 		     off_t offset,
 		     int length     IPSEC_PROC_LAST_ARG)
 {
+	const int max_content = length > 0? length-1 : 0;	/* limit of useful snprintf output */
 	int len = 0;
-	off_t pos = 0, begin = 0;
+	off_t begin = 0;
 	int i;
 	char name[9];
-	struct device *dev, *privdev;
+	struct net_device *dev, *privdev;
 	struct ipsecpriv *priv;
 
 	KLIPS_PRINT(debug_tunnel & DB_TN_PROCFS,
@@ -471,13 +499,13 @@ ipsec_tncfg_get_info(char *buffer,
 		dev = __ipsec_dev_get(name);
 		if(dev) {
 			priv = (struct ipsecpriv *)(dev->priv);
-			len += sprintf(buffer + len, "%s",
+ 			len += ipsec_snprintf(buffer+len, length-len, "%s",
 				       dev->name);
 			if(priv) {
-				privdev = (struct device *)(priv->dev);
-				len += sprintf(buffer + len, " -> %s",
+				privdev = (struct net_device *)(priv->dev);
+ 				len += ipsec_snprintf(buffer+len, length-len, " -> %s",
 					       privdev ? privdev->name : "NULL");
-				len += sprintf(buffer + len, " mtu=%d(%d) -> %d",
+ 				len += ipsec_snprintf(buffer+len, length-len, " mtu=%d(%d) -> %d",
 					       dev->mtu,
 					       priv->mtu,
 					       privdev ? privdev->mtu : 0);
@@ -486,23 +514,27 @@ ipsec_tncfg_get_info(char *buffer,
 					    "klips_debug:ipsec_tncfg_get_info: device '%s' has no private data space!\n",
 					    dev->name);
 			}
-			len += sprintf(buffer + len, "\n");
+			len += ipsec_snprintf(buffer+len, length-len, "\n");
 
-			pos = begin + len;
-			if(pos < offset) {
-				len = 0;
-				begin = pos;
-			}
-			else if (pos > offset + length)	{
+			if (len >= max_content) {
+				/* we've done all that can fit -- stop loop */
+				len = max_content;	/* truncate crap */
 				break;
+			} else {
+				const off_t pos = begin + len;	/* file position of end of what we've generated */
+
+				if (pos <= offset) {
+					/* all is before first interesting character:
+					 * discard, but note where we are.
+					 */
+					len = 0;
+					begin = pos;
+				}
 			}
 		}
 	}
 	*start = buffer + (offset - begin);	/* Start of wanted data */
-	len -= (offset - begin);			/* Start slop */
-	if (len > length)
-		len = length;
-	return len;
+	return len - (offset - begin);
 }
 
 IPSEC_PROCFS_DEBUG_NO_STATIC
@@ -512,8 +544,8 @@ ipsec_version_get_info(char *buffer,
 		       off_t offset,
 		       int length  IPSEC_PROC_LAST_ARG)
 {
-	int len = 0;
-	off_t begin = 0;
+	const int max_content = length > 0? length-1 : 0;	/* limit of useful snprintf output */
+	int len;
 
 	KLIPS_PRINT(debug_tunnel & DB_TN_PROCFS,
 		    "klips_debug:ipsec_version_get_info: "
@@ -523,7 +555,7 @@ ipsec_version_get_info(char *buffer,
 		    (int)offset,
 		    length);
 
-	len += sprintf(buffer + len, "FreeS/WAN version: %s\n",
+	len = ipsec_snprintf(buffer, length, "FreeS/WAN version: %s\n",
 		       ipsec_version_code());
 #if 0
 	KLIPS_PRINT(debug_tunnel & DB_TN_PROCFS,
@@ -544,11 +576,11 @@ ipsec_version_get_info(char *buffer,
 		    radij_c_version);
 #endif
 
-	*start = buffer + (offset - begin);	/* Start of wanted data */
-	len -= (offset - begin);			/* Start slop */
-	if (len > length)
-		len = length;
-	return len;
+	if (len >= max_content)
+		len = max_content;	/* truncate crap */
+
+	*start = buffer + offset;	/* Start of wanted data */
+	return len > offset? len - offset : 0;
 }
 
 IPSEC_PROCFS_DEBUG_NO_STATIC
@@ -607,7 +639,7 @@ ipsec_birth_set(struct file *file, const char *buffer,
 }
 
 
-#ifdef CONFIG_IPSEC_DEBUG
+#ifdef CONFIG_KLIPS_DEBUG
 IPSEC_PROCFS_DEBUG_NO_STATIC
 int
 ipsec_klipsdebug_get_info(char *buffer,
@@ -615,8 +647,8 @@ ipsec_klipsdebug_get_info(char *buffer,
 			  off_t offset,
 			  int length      IPSEC_PROC_LAST_ARG)
 {
+	const int max_content = length > 0? length-1 : 0;	/* limit of useful snprintf output */
 	int len = 0;
-	off_t begin = 0;
 
 	KLIPS_PRINT(debug_tunnel & DB_TN_PROCFS,
 		    "klips_debug:ipsec_klipsdebug_get_info: "
@@ -626,47 +658,45 @@ ipsec_klipsdebug_get_info(char *buffer,
 		    (int)offset,
 		    length);
 
-	len += sprintf(buffer + len, "debug_tunnel=%08x.\n", debug_tunnel);
-	len += sprintf(buffer + len, "debug_netlink=%08x.\n", debug_netlink);
-	len += sprintf(buffer + len, "debug_xform=%08x.\n", debug_xform);
-	len += sprintf(buffer + len, "debug_eroute=%08x.\n", debug_eroute);
-	len += sprintf(buffer + len, "debug_spi=%08x.\n", debug_spi);
-	len += sprintf(buffer + len, "debug_radij=%08x.\n", debug_radij);
-	len += sprintf(buffer + len, "debug_esp=%08x.\n", debug_esp);
-	len += sprintf(buffer + len, "debug_ah=%08x.\n", debug_ah);
-	len += sprintf(buffer + len, "debug_rcv=%08x.\n", debug_rcv);
-	len += sprintf(buffer + len, "debug_pfkey=%08x.\n", debug_pfkey);
+ 	len += ipsec_snprintf(buffer+len, length-len, "debug_tunnel=%08x.\n", debug_tunnel);
+ 	len += ipsec_snprintf(buffer+len, length-len, "debug_xform=%08x.\n", debug_xform);
+ 	len += ipsec_snprintf(buffer+len, length-len, "debug_eroute=%08x.\n", debug_eroute);
+ 	len += ipsec_snprintf(buffer+len, length-len, "debug_spi=%08x.\n", debug_spi);
+ 	len += ipsec_snprintf(buffer+len, length-len, "debug_radij=%08x.\n", debug_radij);
+ 	len += ipsec_snprintf(buffer+len, length-len, "debug_esp=%08x.\n", debug_esp);
+ 	len += ipsec_snprintf(buffer+len, length-len, "debug_rcv=%08x.\n", debug_rcv);
+ 	len += ipsec_snprintf(buffer+len, length-len, "debug_pfkey=%08x.\n", debug_pfkey);
 
-	*start = buffer + (offset - begin);	/* Start of wanted data */
-	len -= (offset - begin);			/* Start slop */
-	if (len > length)
-		len = length;
-	return len;
+	if (len >= max_content)
+		len = max_content;	/* truncate crap */
+
+	*start = buffer + offset;	/* Start of wanted data */
+	return len > offset? len - offset : 0;
 }
-#endif /* CONFIG_IPSEC_DEBUG */
+#endif /* CONFIG_KLIPS_DEBUG */
 
 IPSEC_PROCFS_DEBUG_NO_STATIC
 int
 ipsec_stats_get_int_info(char *buffer,
 			 char **start,
 			 off_t offset,
-			 int count,
-			 int *eof,
-			 void *data)
+			 int   length,
+			 int   *eof,
+			 void  *data)
 {
-	int len = 0;
-	off_t begin = 0;
+	const int max_content = length > 0? length-1 : 0;	/* limit of useful snprintf output */
+	int len;
 	int *thing;
 
 	thing = (int *)data;
 	
-	len += sprintf(buffer + len, "%08x\n", *thing);
+	len = ipsec_snprintf(buffer, length, "%08x\n", *thing);
 
-	*start = buffer + (offset - begin);	/* Start of wanted data */
-	len -= (offset - begin);			/* Start slop */
-	if (len > count)
-		len = count;
-	return len;
+	if (len >= max_content)
+		len = max_content;	/* truncate crap */
+
+	*start = buffer + offset;	/* Start of wanted data */
+	return len > offset? len - offset : 0;
 }
 
 #ifndef PROC_FS_2325
@@ -720,7 +750,7 @@ struct proc_dir_entry ipsec_version =
 	NULL, NULL, NULL, NULL, NULL
 };
 
-#ifdef CONFIG_IPSEC_DEBUG
+#ifdef CONFIG_KLIPS_DEBUG
 struct proc_dir_entry ipsec_klipsdebug =
 {
 	0,
@@ -730,7 +760,7 @@ struct proc_dir_entry ipsec_klipsdebug =
 	ipsec_klipsdebug_get_info,
 	NULL, NULL, NULL, NULL, NULL
 };
-#endif /* CONFIG_IPSEC_DEBUG */
+#endif /* CONFIG_KLIPS_DEBUG */
 #endif /* !PROC_FS_2325 */
 #endif /* CONFIG_PROC_FS */
 
@@ -744,7 +774,7 @@ struct ipsec_proc_list {
 	void                   *data;
 };
 static struct ipsec_proc_list proc_items[]={
-#ifdef CONFIG_IPSEC_DEBUG
+#ifdef CONFIG_KLIPS_DEBUG
 	{"klipsdebug", &proc_net_ipsec_dir, NULL,             ipsec_klipsdebug_get_info, NULL, NULL},
 #endif
 	{"eroute",     &proc_net_ipsec_dir, &proc_eroute_dir, NULL, NULL, NULL},
@@ -787,9 +817,9 @@ ipsec_proc_init()
 	error |= proc_register_dynamic(&proc_net, &ipsec_spigrp);
 	error |= proc_register_dynamic(&proc_net, &ipsec_tncfg);
 	error |= proc_register_dynamic(&proc_net, &ipsec_version);
-#ifdef CONFIG_IPSEC_DEBUG
+#ifdef CONFIG_KLIPS_DEBUG
 	error |= proc_register_dynamic(&proc_net, &ipsec_klipsdebug);
-#endif /* CONFIG_IPSEC_DEBUG */
+#endif /* CONFIG_KLIPS_DEBUG */
 #endif
 
 	/* for 2.2 kernels */
@@ -799,9 +829,9 @@ ipsec_proc_init()
 	error |= proc_register(proc_net, &ipsec_spigrp);
 	error |= proc_register(proc_net, &ipsec_tncfg);
 	error |= proc_register(proc_net, &ipsec_version);
-#ifdef CONFIG_IPSEC_DEBUG
+#ifdef CONFIG_KLIPS_DEBUG
 	error |= proc_register(proc_net, &ipsec_klipsdebug);
-#endif /* CONFIG_IPSEC_DEBUG */
+#endif /* CONFIG_KLIPS_DEBUG */
 #endif
 
 	/* for 2.4 kernels */
@@ -864,11 +894,11 @@ ipsec_proc_cleanup()
 	/* for 2.0 and 2.2 kernels */
 #if !defined(PROC_FS_2325) 
 
-#ifdef CONFIG_IPSEC_DEBUG
+#ifdef CONFIG_KLIPS_DEBUG
 	if (proc_net_unregister(ipsec_klipsdebug.low_ino) != 0)
 		printk("klips_debug:ipsec_cleanup: "
 		       "cannot unregister /proc/net/ipsec_klipsdebug\n");
-#endif /* CONFIG_IPSEC_DEBUG */
+#endif /* CONFIG_KLIPS_DEBUG */
 
 	if (proc_net_unregister(ipsec_version.low_ino) != 0)
 		printk("klips_debug:ipsec_cleanup: "
@@ -906,9 +936,9 @@ ipsec_proc_cleanup()
 	}
 
 
-#ifdef CONFIG_IPSEC_DEBUG
+#ifdef CONFIG_KLIPS_DEBUG
 	remove_proc_entry("ipsec_klipsdebug", proc_net);
-#endif /* CONFIG_IPSEC_DEBUG */
+#endif /* CONFIG_KLIPS_DEBUG */
 	remove_proc_entry("ipsec_eroute",     proc_net);
 	remove_proc_entry("ipsec_spi",        proc_net);
 	remove_proc_entry("ipsec_spigrp",     proc_net);
@@ -920,6 +950,52 @@ ipsec_proc_cleanup()
 
 /*
  * $Log: ipsec_proc.c,v $
+ * Revision 1.36  2004/04/12 19:48:31  dhr
+ *
+ * convert more sprintf calls to ipsec_snprintf and then fix surrounding code
+ *
+ * Revision 1.35  2004/04/12 17:39:05  dhr
+ *
+ * simplify and clarify struct wsbuf (used in generating /proc/net/ipsec_eroute)
+ *
+ * Revision 1.34  2004/04/12 17:36:19  dhr
+ *
+ * refine handling buffer end in generating /proc/net/ipsec_eroute
+ *
+ * Revision 1.33  2004/04/12 04:46:35  sam
+ * - ipsec_snprintf returns 0 length string when vsnprintf returns error (because
+ *   callers of ipsec_snprintf don't handle an error return)
+ * - in ipsec_eroute_get_info, fix case offset != w.begin
+ * - in ipsec_rj_walker_procprint, remove scar tissue.
+ * - in ipsec_rj_walker_procprint, be handle the fact that an overflowing snprintf
+ *   jams a NUL at the end of the buffer.
+ *
+ * Revision 1.32  2004/03/24 01:17:51  mcr
+ * 	fixes for sprintf() problem.
+ *
+ * Revision 1.31  2004/02/24 17:17:04  mcr
+ * 	s/CONFIG_IPSEC/CONFIG_KLIPS/ as 26sec uses "CONFIG_IPSEC" to
+ * 	turn it on/off as well.
+ *
+ * Revision 1.30  2004/02/22 06:50:42  mcr
+ * 	kernel 2.6 port - merged with 2.4 code.
+ *
+ * Revision 1.29.6.1  2004/02/20 14:10:18  mcr
+ * 	moved code to net/ipsec/ to make 2.6 happy.
+ *
+ * Revision 1.29  2003/12/13 04:09:21  mcr
+ * 	AH transform removed.
+ *
+ * Revision 1.28  2003/12/06 21:21:38  mcr
+ * 	split up receive path into per-transform files, for
+ * 	easier later removal.
+ *
+ * Revision 1.27  2003/12/04 19:05:54  mcr
+ * 	cleaned up "sa_id" structure to use "ip_said" only.
+ *
+ * Revision 1.26  2003/11/07 17:37:12  mcr
+ * 	backout X.509 and port-selector code
+ *
  * Revision 1.24  2003/06/20 01:42:21  mcr
  * 	added counters to measure how many ACQUIREs we send to pluto,
  * 	and how many are successfully sent.
