@@ -14,7 +14,7 @@
  * for more details.
  */
 
-char ipsec_rcv_c_version[] = "RCSID $Id: ipsec_rcv.c,v 1.132 2003/09/02 19:51:48 mcr Exp $";
+char ipsec_rcv_c_version[] = "RCSID $Id: ipsec_rcv.c,v 1.141 2004/02/24 17:17:04 mcr Exp $";
 
 #include <linux/config.h>
 #include <linux/version.h>
@@ -59,44 +59,37 @@ char ipsec_rcv_c_version[] = "RCSID $Id: ipsec_rcv.c,v 1.132 2003/09/02 19:51:48
 #include "freeswan/ipsec_sa.h"
 
 #include "freeswan/ipsec_radij.h"
-#include "freeswan/ipsec_netlink.h"
 #include "freeswan/ipsec_xform.h"
 #include "freeswan/ipsec_tunnel.h"
 #include "freeswan/ipsec_rcv.h"
 
-#if defined(CONFIG_IPSEC_ESP) || defined(CONFIG_IPSEC_AH)
-#include "freeswan/ipsec_ah.h"
-#endif /* defined(CONFIG_IPSEC_ESP) || defined(CONFIG_IPSEC_AH) */
+#include "freeswan/ipsec_auth.h"
 
-#ifdef CONFIG_IPSEC_ESP
 #include "freeswan/ipsec_esp.h"
-#endif /* !CONFIG_IPSEC_ESP */
 
-#ifdef CONFIG_IPSEC_IPCOMP
-#include "freeswan/ipcomp.h"
-#endif /* CONFIG_IPSEC_COMP */
+#ifdef CONFIG_KLIPS_IPCOMP
+#include "freeswan/ipsec_ipcomp.h"
+#endif /* CONFIG_KLIPS_COMP */
 
 #include <pfkeyv2.h>
 #include <pfkey.h>
 
 #include "freeswan/ipsec_proto.h"
 
-#ifdef CONFIG_IPSEC_DEBUG
-int debug_ah = 0;
-int debug_esp = 0;
+#ifdef CONFIG_KLIPS_DEBUG
 int debug_rcv = 0;
-#endif /* CONFIG_IPSEC_DEBUG */
+#endif /* CONFIG_KLIPS_DEBUG */
 
 int sysctl_ipsec_inbound_policy_check = 1;
 
-#ifdef CONFIG_IPSEC_DEBUG
-static void
-rcv_dmp(char *s, caddr_t bb, int len)
+#ifdef CONFIG_KLIPS_DEBUG
+void
+ipsec_dmp(char *s, caddr_t bb, int len)
 {
 	int i;
 	unsigned char *b = bb;
   
-	if (debug_rcv && sysctl_ipsec_debug_verbose) {
+
 		printk(KERN_INFO "klips_debug:ipsec_tunnel_:dmp: "
 		       "at %s, len=%d:",
 		       s,
@@ -108,16 +101,8 @@ rcv_dmp(char *s, caddr_t bb, int len)
 			printk(" %02x", *b++);
 		}
 		printk("\n");
-	}
 }
-#else /* CONFIG_IPSEC_DEBUG */
-#define rcv_dmp(_x, _y, _z) 
-#endif /* CONFIG_IPSEC_DEBUG */
-
-
-#if defined(CONFIG_IPSEC_ESP) || defined(CONFIG_IPSEC_AH)
-__u32 zeroes[AH_AMAX];
-#endif /* defined(CONFIG_IPSEC_ESP) || defined(CONFIG_IPSEC_AH) */
+#endif /* CONFIG_KLIPS_DEBUG */
 
 /*
  * Check-replay-window routine, adapted from the original
@@ -199,645 +184,21 @@ ipsec_updatereplaywindow(struct ipsec_sa*ipsp, __u32 seq)
 	return 1;			/* out of order but good */
 }
 
-#ifdef CONFIG_IPSEC_AUTH_HMAC_MD5
+#ifdef CONFIG_KLIPS_AUTH_HMAC_MD5
 struct auth_alg ipsec_rcv_md5[]={
 	{MD5Init, MD5Update, MD5Final, AHMD596_ALEN}
 };
 
-#endif /* CONFIG_IPSEC_AUTH_HMAC_MD5 */
+#endif /* CONFIG_KLIPS_AUTH_HMAC_MD5 */
 
-#ifdef CONFIG_IPSEC_AUTH_HMAC_SHA1
+#ifdef CONFIG_KLIPS_AUTH_HMAC_SHA1
 struct auth_alg ipsec_rcv_sha1[]={
 	{SHA1Init, SHA1Update, SHA1Final, AHSHA196_ALEN}
 };
-#endif /* CONFIG_IPSEC_AUTH_HMAC_MD5 */
-
-enum ipsec_rcv_value {
-	IPSEC_RCV_LASTPROTO=1,
-	IPSEC_RCV_OK=0,
-	IPSEC_RCV_BADPROTO=-1,
-	IPSEC_RCV_BADLEN=-2,
-	IPSEC_RCV_ESP_BADALG=-3,
-	IPSEC_RCV_3DES_BADBLOCKING=-4,
-	IPSEC_RCV_ESP_DECAPFAIL=-5,
-	IPSEC_RCV_DECAPFAIL=-6,
-	IPSEC_RCV_SAIDNOTFOUND=-7,
-	IPSEC_RCV_IPCOMPALONE=-8,
-	IPSEC_RCV_IPCOMPFAILED=-10,
-	IPSEC_RCV_SAIDNOTLIVE=-11,
-	IPSEC_RCV_FAILEDINBOUND=-12,
-	IPSEC_RCV_LIFETIMEFAILED=-13,
-	IPSEC_RCV_BADAUTH=-14,
-	IPSEC_RCV_REPLAYFAILED=-15,
-	IPSEC_RCV_AUTHFAILED=-16,
-	IPSEC_RCV_REPLAYROLLED=-17
-};
-
-struct ipsec_rcv_state {
-	struct sk_buff *skb;
-	struct net_device_stats *stats;
-	struct iphdr *ipp;
-	struct ipsec_sa *ipsp;
-	int len;
-	int ilen;
-	int authlen;
-	int hard_header_len;
-	int iphlen;
-	struct auth_alg *authfuncs;
-	struct sa_id said;
-	char   sa[SATOA_BUF];
-	size_t sa_len;
-	__u8 next_header;
-	__u8 hash[AH_AMAX];
-	char ipsaddr_txt[ADDRTOA_BUF];
-	char ipdaddr_txt[ADDRTOA_BUF];
-	__u8 *octx;
-	__u8 *ictx;
-	int ictx_len;
-	int octx_len;
-	union {
-		struct {
-			struct esphdr *espp;
-		} espstuff;
-		struct {
-			struct ahhdr *ahp;
-		} ahstuff;
-		struct {
-			struct ipcomphdr *compp;
-		} ipcompstuff;
-	} protostuff;
-};
-
-struct xform_functions {
-	enum ipsec_rcv_value (*checks)(struct ipsec_rcv_state *irs,
-				       struct sk_buff *skb);
-        enum ipsec_rcv_value (*decrypt)(struct ipsec_rcv_state *irs);
-
-	enum ipsec_rcv_value (*setup_auth)(struct ipsec_rcv_state *irs,
-					   struct sk_buff *skb,
-					   __u32          *replay,
-					   unsigned char **authenticator);
-	enum ipsec_rcv_value (*calc_auth)(struct ipsec_rcv_state *irs,
-					struct sk_buff *skb);
-};
-
-#ifdef CONFIG_IPSEC_ESP
-enum ipsec_rcv_value
-ipsec_rcv_esp_checks(struct ipsec_rcv_state *irs,
-		     struct sk_buff *skb)
-{
-	__u8 proto;
-	int len;	/* packet length */
-
-	len = skb->len;
-	proto = irs->ipp->protocol;
-
-	/* XXX this will need to be 8 for IPv6 */
-	if ((proto == IPPROTO_ESP) && ((len - irs->iphlen) % 4)) {
-		printk("klips_error:ipsec_rcv: "
-		       "got packet with content length = %d from %s -- should be on 4 octet boundary, packet dropped\n",
-		       len - irs->iphlen,
-		       irs->ipsaddr_txt);
-		if(irs->stats) {
-			irs->stats->rx_errors++;
-		}
-		return IPSEC_RCV_BADLEN;
-	}
-
-	if(skb->len < (irs->hard_header_len + sizeof(struct iphdr) + sizeof(struct esphdr))) {
-		KLIPS_PRINT(debug_rcv & DB_RX_INAU,
-			    "klips_debug:ipsec_rcv: "
-			    "runt esp packet of skb->len=%d received from %s, dropped.\n",
-			    skb->len,
-			    irs->ipsaddr_txt);
-		if(irs->stats) {
-			irs->stats->rx_errors++;
-		}
-		return IPSEC_RCV_BADLEN;
-	}
-
-	irs->protostuff.espstuff.espp = (struct esphdr *)(skb->data + irs->iphlen);
-	irs->said.spi = irs->protostuff.espstuff.espp->esp_spi;
-
-	return IPSEC_RCV_OK;
-}
+#endif /* CONFIG_KLIPS_AUTH_HMAC_MD5 */
 
 enum ipsec_rcv_value
-ipsec_rcv_esp_decrypt_setup(struct ipsec_rcv_state *irs,
-			    struct sk_buff *skb,
-			    __u32          *replay,
-			    unsigned char **authenticator)
-{
-	struct esphdr *espp = irs->protostuff.espstuff.espp;
-
-	KLIPS_PRINT(debug_rcv,
-		    "klips_debug:ipsec_rcv: "
-		    "packet from %s received with seq=%d (iv)=0x%08x%08x iplen=%d esplen=%d sa=%s\n",
-		    irs->ipsaddr_txt,
-		    (__u32)ntohl(espp->esp_rpl),
-		    (__u32)ntohl(*((__u32 *)(espp->esp_iv)    )),
-		    (__u32)ntohl(*((__u32 *)(espp->esp_iv) + 1)),
-		    irs->len,
-		    irs->ilen,
-		    irs->sa_len ? irs->sa : " (error)");
-
-	*replay = ntohl(espp->esp_rpl);
-	*authenticator = &(skb->data[irs->len - irs->authlen]);
-
-	return IPSEC_RCV_OK;
-}
-
-enum ipsec_rcv_value
-ipsec_rcv_esp_authcalc(struct ipsec_rcv_state *irs,
-		       struct sk_buff *skb)
-{
-	struct auth_alg *aa;
-	struct esphdr *espp = irs->protostuff.espstuff.espp;
-	union {
-		MD5_CTX		md5;
-		SHA1_CTX	sha1;
-	} tctx;
-
-	aa = irs->authfuncs;
-
-	/* copy the initialized keying material */
-	memcpy(&tctx, irs->ictx, irs->ictx_len);
-
-	(*aa->update)((void *)&tctx, (caddr_t)espp, irs->ilen);
-
-	(*aa->final)(irs->hash, (void *)&tctx);
-
-	memcpy(&tctx, irs->octx, irs->octx_len);
-
-	(*aa->update)((void *)&tctx, irs->hash, aa->hashlen);
-	(*aa->final)(irs->hash, (void *)&tctx);
-
-	return IPSEC_RCV_OK;
-}
-
-
-enum ipsec_rcv_value
-ipsec_rcv_esp_decrypt(struct ipsec_rcv_state *irs)
-{
-	struct ipsec_sa *ipsp = irs->ipsp;
-	struct esphdr *espp = irs->protostuff.espstuff.espp;
-	int esphlen = 0;
-	__u8 *idat;	/* pointer to content to be decrypted/authenticated */
-	__u32 iv[2];
-	int pad = 0, padlen;
-	int badpad = 0;
-	int i;
-	struct sk_buff *skb;
-
-	skb=irs->skb;
-
-	idat = skb->data + irs->iphlen;
-
-	switch(ipsp->ips_encalg) {
-	case ESP_3DES:
-		iv[0] = *((__u32 *)(espp->esp_iv)    );
-		iv[1] = *((__u32 *)(espp->esp_iv) + 1);
-		esphlen = sizeof(struct esphdr);
-		break;
-	default:
-		ipsp->ips_errs.ips_alg_errs += 1;
-		if(irs->stats) {
-			irs->stats->rx_errors++;
-		}
-		return IPSEC_RCV_ESP_BADALG;
-	}
-
-	idat += esphlen;
-	irs->ilen -= esphlen;
-
-	switch(ipsp->ips_encalg) {
-	case ESP_3DES:
-		if ((irs->ilen) % 8) {
-			ipsp->ips_errs.ips_encsize_errs += 1;
-			printk("klips_error:ipsec_rcv: "
-			       "got packet with esplen = %d from %s -- should be on 8 octet boundary, packet dropped\n",
-			       irs->ilen,
-			       irs->ipsaddr_txt);
-			if(irs->stats) {
-				irs->stats->rx_errors++;
-			}
-			return IPSEC_RCV_3DES_BADBLOCKING;
-		}
-		des_ede3_cbc_encrypt((des_cblock *)idat,
-				     (des_cblock *)idat,
-				     irs->ilen,
-				     ((struct des_eks *)(ipsp->ips_key_e))[0].ks,
-				     ((struct des_eks *)(ipsp->ips_key_e))[1].ks,
-				     ((struct des_eks *)(ipsp->ips_key_e))[2].ks,
-				     (des_cblock *)iv, 0);
-		break;
-	}
-
-	rcv_dmp("postdecrypt", skb->data, skb->len);
-
-	irs->next_header = idat[irs->ilen - 1];
-	padlen = idat[irs->ilen - 2];
-	pad = padlen + 2 + irs->authlen;
-
-	KLIPS_PRINT(debug_rcv & DB_RX_IPAD,
-		    "klips_debug:ipsec_rcv: "
-		    "padlen=%d, contents: 0x<offset>: 0x<value> 0x<value> ...\n",
-		    padlen);
-
-	for (i = 1; i <= padlen; i++) {
-		if((i % 16) == 1) {
-			KLIPS_PRINT(debug_rcv & DB_RX_IPAD,
-				    "klips_debug:           %02x:",
-				    i - 1);
-		}
-		KLIPS_PRINTMORE(debug_rcv & DB_RX_IPAD,
-				" %02x",
-				idat[irs->ilen - 2 - padlen + i - 1]);
-		if(i != idat[irs->ilen - 2 - padlen + i - 1]) {
-			badpad = 1;
-		}
-		if((i % 16) == 0) {
-			KLIPS_PRINTMORE(debug_rcv & DB_RX_IPAD,
-					"\n");
-		}
-	}
-	if((i % 16) != 1) {
-		KLIPS_PRINTMORE(debug_rcv & DB_RX_IPAD,
-						"\n");
-	}
-	if(badpad) {
-		KLIPS_PRINT(debug_rcv & DB_RX_IPAD,
-			    "klips_debug:ipsec_rcv: "
-			    "warning, decrypted packet from %s has bad padding\n",
-			    irs->ipsaddr_txt);
-		KLIPS_PRINT(debug_rcv & DB_RX_IPAD,
-			    "klips_debug:ipsec_rcv: "
-			    "...may be bad decryption -- not dropped\n");
-		ipsp->ips_errs.ips_encpad_errs += 1;
-	}
-
-	KLIPS_PRINT(debug_rcv & DB_RX_IPAD,
-		    "klips_debug:ipsec_rcv: "
-		    "packet decrypted from %s: next_header = %d, padding = %d\n",
-		    irs->ipsaddr_txt,
-		    irs->next_header,
-		    pad - 2 - irs->authlen);
-
-	irs->ipp->tot_len = htons(ntohs(irs->ipp->tot_len) - (esphlen + pad));
-
-	/*
-	 * move the IP header forward by the size of the ESP header, which
-	 * will remove the the ESP header from the packet.
-	 */
-	memmove((void *)(skb->data + esphlen),
-		(void *)(skb->data), irs->iphlen);
-
-	rcv_dmp("esp postmove", skb->data, skb->len);
-
-	/* skb_pull below, will move up by esphlen */
-
-	/* XXX not clear how this can happen, as the message indicates */
-	if(skb->len < esphlen) {
-		printk(KERN_WARNING
-		       "klips_error:ipsec_rcv: "
-		       "tried to skb_pull esphlen=%d, %d available.  This should never happen, please report.\n",
-		       esphlen, (int)(skb->len));
-		return IPSEC_RCV_ESP_DECAPFAIL;
-	}
-	skb_pull(skb, esphlen);
-
-	irs->ipp = (struct iphdr *)skb->data;
-
-	rcv_dmp("esp postpull", skb->data, skb->len);
-
-	/* now, trip off the padding from the end */
-	KLIPS_PRINT(debug_rcv & DB_RX_PKTRX,
-		    "klips_debug:ipsec_rcv: "
-		    "trimming to %d.\n",
-		    irs->len - esphlen - pad);
-	if(pad + esphlen <= irs->len) {
-		skb_trim(skb, irs->len - esphlen - pad);
-	} else {
-		KLIPS_PRINT(debug_rcv & DB_RX_PKTRX,
-			    "klips_debug:ipsec_rcv: "
-			    "bogus packet, size is zero or negative, dropping.\n");
-		return IPSEC_RCV_DECAPFAIL;
-	}
-
-	return IPSEC_RCV_OK;
-}
-
-
-struct xform_functions esp_rcv_funcs[]={
-	{	checks:         ipsec_rcv_esp_checks,
-		setup_auth:     ipsec_rcv_esp_decrypt_setup,
-		calc_auth:      ipsec_rcv_esp_authcalc,
-		decrypt:        ipsec_rcv_esp_decrypt,
-	},
-};
-#endif /* !CONFIG_IPSEC_ESP */
-
-#ifdef CONFIG_IPSEC_AH
-enum ipsec_rcv_value
-ipsec_rcv_ah_checks(struct ipsec_rcv_state *irs,
-		    struct sk_buff *skb)
-{
-	int ahminlen;
-
-	ahminlen = irs->hard_header_len + sizeof(struct iphdr);
-
-	/* take care not to deref this pointer until we check the minlen though */
-	irs->protostuff.ahstuff.ahp = (struct ahhdr *) (skb->data + irs->iphlen);
-
-	if((skb->len < ahminlen+sizeof(struct ahhdr)) ||
-	   (skb->len < ahminlen+(irs->protostuff.ahstuff.ahp->ah_hl << 2))) {
-		KLIPS_PRINT(debug_rcv & DB_RX_INAU,
-			    "klips_debug:ipsec_rcv: "
-			    "runt ah packet of skb->len=%d received from %s, dropped.\n",
-			    skb->len,
-			    irs->ipsaddr_txt);
-		if(irs->stats) {
-			irs->stats->rx_errors++;
-		}
-		return IPSEC_RCV_BADLEN;
-	}
-
-	irs->said.spi = irs->protostuff.ahstuff.ahp->ah_spi;
-
-	/* XXX we only support the one 12-byte authenticator for now */
-	if(irs->protostuff.ahstuff.ahp->ah_hl != ((AHHMAC_HASHLEN+AHHMAC_RPLLEN) >> 2)) {
-		KLIPS_PRINT(debug_rcv & DB_RX_INAU,
-			    "klips_debug:ipsec_rcv: "
-			    "bad authenticator length %ld, expected %lu from %s.\n",
-			    (long)(irs->protostuff.ahstuff.ahp->ah_hl << 2),
-			    (unsigned long) sizeof(struct ahhdr),
-			    irs->ipsaddr_txt);
-		if(irs->stats) {
-			irs->stats->rx_errors++;
-		}
-		return IPSEC_RCV_BADLEN;
-	}
-
-	return IPSEC_RCV_OK;
-}
-
-
-enum ipsec_rcv_value
-ipsec_rcv_ah_setup_auth(struct ipsec_rcv_state *irs,
-			struct sk_buff *skb,
-			__u32          *replay,
-			unsigned char **authenticator)
-{
-	struct ahhdr *ahp = irs->protostuff.ahstuff.ahp;
-
-	*replay = ntohl(ahp->ah_rpl);
-	*authenticator = ahp->ah_data;
-
-	return IPSEC_RCV_OK;
-}
-
-enum ipsec_rcv_value
-ipsec_rcv_ah_authcalc(struct ipsec_rcv_state *irs,
-		      struct sk_buff *skb)
-{
-	struct auth_alg *aa;
-	struct ahhdr *ahp = irs->protostuff.ahstuff.ahp;
-	union {
-		MD5_CTX		md5;
-		SHA1_CTX	sha1;
-	} tctx;
-	struct iphdr ipo;
-	int ahhlen;
-
-	aa = irs->authfuncs;
-
-	/* copy the initialized keying material */
-	memcpy(&tctx, irs->ictx, irs->ictx_len);
-
-	ipo = *irs->ipp;
-	ipo.tos = 0;	/* mutable RFC 2402 3.3.3.1.1.1 */
-	ipo.frag_off = 0;
-	ipo.ttl = 0;
-	ipo.check = 0;
-
-
-	/* do the sanitized header */
-	(*aa->update)((void*)&tctx, (caddr_t)&ipo, sizeof(struct iphdr));
-
-	/* XXX we didn't do the options here! */
-
-	/* now do the AH header itself */
-	ahhlen = AH_BASIC_LEN + (ahp->ah_hl << 2);
-	(*aa->update)((void*)&tctx, (caddr_t)ahp,  ahhlen - AHHMAC_HASHLEN);
-
-	/* now, do some zeroes */
-	(*aa->update)((void*)&tctx, (caddr_t)zeroes,  AHHMAC_HASHLEN);
-
-	/* finally, do the packet contents themselves */
-	(*aa->update)((void*)&tctx,
-		      (caddr_t)skb->data + irs->iphlen + ahhlen,
-		      skb->len - irs->iphlen - ahhlen);
-
-	(*aa->final)(irs->hash, (void *)&tctx);
-
-	memcpy(&tctx, irs->octx, irs->octx_len);
-
-	(*aa->update)((void *)&tctx, irs->hash, aa->hashlen);
-	(*aa->final)(irs->hash, (void *)&tctx);
-
-	return IPSEC_RCV_OK;
-}
-
-enum ipsec_rcv_value
-ipsec_rcv_ah_decap(struct ipsec_rcv_state *irs)
-{
-	struct ahhdr *ahp = irs->protostuff.ahstuff.ahp;
-	struct sk_buff *skb;
-	int ahhlen;
-
-	skb=irs->skb;
-
-	ahhlen = AH_BASIC_LEN + (ahp->ah_hl << 2);
-
-	irs->ipp->tot_len = htons(ntohs(irs->ipp->tot_len) - ahhlen);
-	irs->next_header  = ahp->ah_nh;
-
-	/*
-	 * move the IP header forward by the size of the AH header, which
-	 * will remove the the AH header from the packet.
-	 */
-	memmove((void *)(skb->data + ahhlen),
-		(void *)(skb->data), irs->iphlen);
-
-	rcv_dmp("ah postmove", skb->data, skb->len);
-
-	/* skb_pull below, will move up by ahhlen */
-
-	/* XXX not clear how this can happen, as the message indicates */
-	if(skb->len < ahhlen) {
-		printk(KERN_WARNING
-		       "klips_error:ipsec_rcv: "
-		       "tried to skb_pull ahhlen=%d, %d available.  This should never happen, please report.\n",
-		       ahhlen,
-		       (int)(skb->len));
-		return IPSEC_RCV_DECAPFAIL;
-	}
-	skb_pull(skb, ahhlen);
-
-	irs->ipp = (struct iphdr *)skb->data;
-
-	rcv_dmp("ah postpull", skb->data, skb->len);
-
-	return IPSEC_RCV_OK;
-}
-
-
-struct xform_functions ah_rcv_funcs[]={
-	{	checks:         ipsec_rcv_ah_checks,
-		setup_auth:     ipsec_rcv_ah_setup_auth,
-		calc_auth:      ipsec_rcv_ah_authcalc,
-		decrypt:        ipsec_rcv_ah_decap,
-	},
-};
-
-#endif /* CONFIG_IPSEC_AH */
-
-#ifdef CONFIG_IPSEC_IPCOMP
-enum ipsec_rcv_value
-ipsec_rcv_ipcomp_checks(struct ipsec_rcv_state *irs,
-			struct sk_buff *skb)
-{
-	int ipcompminlen;
-
-	ipcompminlen = irs->hard_header_len + sizeof(struct iphdr);
-
-	if(skb->len < (ipcompminlen + sizeof(struct ipcomphdr))) {
-		KLIPS_PRINT(debug_rcv & DB_RX_INAU,
-			    "klips_debug:ipsec_rcv: "
-			    "runt comp packet of skb->len=%d received from %s, dropped.\n",
-			    skb->len,
-			    irs->ipsaddr_txt);
-		if(irs->stats) {
-			irs->stats->rx_errors++;
-		}
-		return IPSEC_RCV_BADLEN;
-	}
-
-	irs->protostuff.ipcompstuff.compp = (struct ipcomphdr *)(skb->data + irs->iphlen);
-	irs->said.spi = htonl((__u32)ntohs(irs->protostuff.ipcompstuff.compp->ipcomp_cpi));
-	return IPSEC_RCV_OK;
-}
-
-enum ipsec_rcv_value
-ipsec_rcv_ipcomp_decomp(struct ipsec_rcv_state *irs)
-{
-	unsigned int flags = 0;
-	struct ipsec_sa *ipsp = irs->ipsp;
-	struct sk_buff *skb;
-
-	skb=irs->skb;
-
-	rcv_dmp("ipcomp", skb->data, skb->len);
-
-	if(ipsp == NULL) {
-		return IPSEC_RCV_SAIDNOTFOUND;
-	}
-
-#if 0
-	/* we want to check that this wasn't the first SA on the list, because
-	 * we don't support bare IPCOMP, for unexplained reasons. MCR
-	 */
-	if (ipsp->ips_onext != NULL) {
-		KLIPS_PRINT(debug_rcv,
-			    "klips_debug:ipsec_rcv: "
-			    "Incoming packet with outer IPCOMP header SA:%s: not yet supported by KLIPS, dropped\n",
-			    irs->sa_len ? irs->sa : " (error)");
-		if(irs->stats) {
-			irs->stats->rx_dropped++;
-		}
-
-		return IPSEC_RCV_IPCOMPALONE;
-	}
-#endif
-
-	if(sysctl_ipsec_inbound_policy_check &&
-	   ((((ntohl(ipsp->ips_said.spi) & 0x0000ffff) != ntohl(irs->said.spi)) &&
-	     (ipsp->ips_encalg != ntohl(irs->said.spi))   /* this is a workaround for peer non-compliance with rfc2393 */
-		    ))) {
-		char sa2[SATOA_BUF];
-		size_t sa_len2 = 0;
-
-		sa_len2 = satoa(ipsp->ips_said, 0, sa2, SATOA_BUF);
-
-		KLIPS_PRINT(debug_rcv,
-			    "klips_debug:ipsec_rcv: "
-			    "Incoming packet with SA(IPCA):%s does not match policy SA(IPCA):%s cpi=%04x cpi->spi=%08x spi=%08x, spi->cpi=%04x for SA grouping, dropped.\n",
-			    irs->sa_len ? irs->sa : " (error)",
-			    ipsp != NULL ? (sa_len2 ? sa2 : " (error)") : "NULL",
-			    ntohs(irs->protostuff.ipcompstuff.compp->ipcomp_cpi),
-			    (__u32)ntohl(irs->said.spi),
-			    ipsp != NULL ? (__u32)ntohl((ipsp->ips_said.spi)) : 0,
-			    ipsp != NULL ? (__u16)(ntohl(ipsp->ips_said.spi) & 0x0000ffff) : 0);
-		if(irs->stats) {
-			irs->stats->rx_dropped++;
-		}
-		return IPSEC_RCV_SAIDNOTFOUND;
-	}
-
-	ipsp->ips_comp_ratio_cbytes += ntohs(irs->ipp->tot_len);
-	irs->next_header = irs->protostuff.ipcompstuff.compp->ipcomp_nh;
-
-	skb = skb_decompress(skb, ipsp, &flags);
-	if (!skb || flags) {
-		spin_unlock(&tdb_lock);
-		KLIPS_PRINT(debug_rcv,
-			    "klips_debug:ipsec_rcv: "
-			    "skb_decompress() returned error flags=%x, dropped.\n",
-			    flags);
-		if (irs->stats) {
-			if (flags)
-				irs->stats->rx_errors++;
-			else
-				irs->stats->rx_dropped++;
-		}
-		return IPSEC_RCV_IPCOMPFAILED;
-	}
-
-	/* make sure we update the pointer */
-	irs->skb = skb;
-	
-#ifdef NET_21
-	irs->ipp = skb->nh.iph;
-#else /* NET_21 */
-	irs->ipp = skb->ip_hdr;
-#endif /* NET_21 */
-
-	ipsp->ips_comp_ratio_dbytes += ntohs(irs->ipp->tot_len);
-
-	KLIPS_PRINT(debug_rcv,
-		    "klips_debug:ipsec_rcv: "
-		    "packet decompressed SA(IPCA):%s cpi->spi=%08x spi=%08x, spi->cpi=%04x, nh=%d.\n",
-		    irs->sa_len ? irs->sa : " (error)",
-		    (__u32)ntohl(irs->said.spi),
-		    ipsp != NULL ? (__u32)ntohl((ipsp->ips_said.spi)) : 0,
-		    ipsp != NULL ? (__u16)(ntohl(ipsp->ips_said.spi) & 0x0000ffff) : 0,
-		    irs->next_header);
-	KLIPS_IP_PRINT(debug_rcv & DB_RX_PKTRX, irs->ipp);
-
-	return IPSEC_RCV_OK;
-}
-
-
-struct xform_functions ipcomp_rcv_funcs[]={
-	{checks:  ipsec_rcv_ipcomp_checks,
-	 decrypt: ipsec_rcv_ipcomp_decomp,
-	},
-};
-
-#endif /* CONFIG_IPSEC_IPCOMP */
-
-enum ipsec_rcv_value
-ipsec_rcv_decap_once(struct ipsec_rcv_state *irs)
+ipsec_rcv_decap_once(struct ipsec_rcv_state *irs, struct xform_functions *proto_funcs)
 {
 	int iphlen;
 	unsigned char *dat;
@@ -846,7 +207,6 @@ ipsec_rcv_decap_once(struct ipsec_rcv_state *irs)
 	struct in_addr ipdaddr;
 	int replay = 0;	/* replay value in AH or ESP packet */
 	struct ipsec_sa* ipsnext = NULL;	/* next SA towards inside of packet */
-	struct xform_functions *proto_funcs;
 	struct ipsec_sa *newipsp;
 	struct iphdr *ipp;
 	struct sk_buff *skb;
@@ -870,53 +230,32 @@ ipsec_rcv_decap_once(struct ipsec_rcv_state *irs)
 		    "decap (%d) from %s -> %s\n",
 		    proto, irs->ipsaddr_txt, irs->ipdaddr_txt);
 
-	switch(proto) {
-#ifdef CONFIG_IPSEC_ESP
-	case IPPROTO_ESP:
-		proto_funcs = esp_rcv_funcs;
-		break;
-#endif /* !CONFIG_IPSEC_ESP */
-
-#ifdef CONFIG_IPSEC_AH
-	case IPPROTO_AH:
-		proto_funcs = ah_rcv_funcs;
-		break;
-#endif /* !CONFIG_IPSEC_AH */
-
-#ifdef CONFIG_IPSEC_IPCOMP
-	case IPPROTO_COMP:
-		proto_funcs = ipcomp_rcv_funcs;
-		break;
-#endif /* !CONFIG_IPSEC_IPCOMP */
-	default:
-		if(irs->stats) {
-			irs->stats->rx_errors++;
-		}
-		return IPSEC_RCV_BADPROTO;
-	}
-
 	/*
 	 * Find tunnel control block and (indirectly) call the
 	 * appropriate tranform routine. The resulting sk_buf
 	 * is a valid IP packet ready to go through input processing.
 	 */
 
-	irs->said.dst.s_addr = ipp->daddr;
+	irs->said.dst.u.v4.sin_addr.s_addr = ipp->daddr;
+	irs->said.dst.u.v4.sin_family = AF_INET;
+	irs->said.proto = proto;
 
-	if(proto_funcs->checks) {
-		enum ipsec_rcv_value retval = (*proto_funcs->checks)(irs, skb);
+	/* note the rcv_checks fills in the SPI value of the said.*/
+	if(proto_funcs->rcv_checks) {
+		enum ipsec_rcv_value retval =
+		  (*proto_funcs->rcv_checks)(irs, skb);
 
 		if(retval < 0) {
 			return retval;
 		}
 	}
 
-	irs->said.proto = proto;
-	irs->sa_len = satoa(irs->said, 0, irs->sa, SATOA_BUF);
+	irs->sa_len = satot(&irs->said, 0, irs->sa, sizeof(irs->sa));
 	if(irs->sa_len == 0) {
 		strcpy(irs->sa, "(error)");
 	}
 
+
 	newipsp = ipsec_sa_getbyid(&irs->said);
 	if (newipsp == NULL) {
 		KLIPS_PRINT(debug_rcv,
@@ -1046,7 +385,7 @@ ipsec_rcv_decap_once(struct ipsec_rcv_state *irs)
 	irs->authfuncs=NULL;
 	/* authenticate, if required */
 	switch(irs->ipsp->ips_authalg) {
-#ifdef CONFIG_IPSEC_AUTH_HMAC_MD5
+#ifdef CONFIG_KLIPS_AUTH_HMAC_MD5
 	case AH_MD5:
 		irs->authlen = AHHMAC_HASHLEN;
 		irs->authfuncs = ipsec_rcv_md5;
@@ -1055,8 +394,8 @@ ipsec_rcv_decap_once(struct ipsec_rcv_state *irs)
 		irs->ictx_len = sizeof(((struct md5_ctx*)(irs->ipsp->ips_key_a))->ictx);
 		irs->octx_len = sizeof(((struct md5_ctx*)(irs->ipsp->ips_key_a))->octx);
 		break;
-#endif /* CONFIG_IPSEC_AUTH_HMAC_MD5 */
-#ifdef CONFIG_IPSEC_AUTH_HMAC_SHA1
+#endif /* CONFIG_KLIPS_AUTH_HMAC_MD5 */
+#ifdef CONFIG_KLIPS_AUTH_HMAC_SHA1
 	case AH_SHA:
 		irs->authlen = AHHMAC_HASHLEN;
 		irs->authfuncs = ipsec_rcv_sha1;
@@ -1065,7 +404,7 @@ ipsec_rcv_decap_once(struct ipsec_rcv_state *irs)
 		irs->ictx_len = sizeof(((struct sha1_ctx*)(irs->ipsp->ips_key_a))->ictx);
 		irs->octx_len = sizeof(((struct sha1_ctx*)(irs->ipsp->ips_key_a))->octx);
 		break;
-#endif /* CONFIG_IPSEC_AUTH_HMAC_SHA1 */
+#endif /* CONFIG_KLIPS_AUTH_HMAC_SHA1 */
 	case AH_NONE:
 		irs->authlen = 0;
 		break;
@@ -1092,9 +431,9 @@ ipsec_rcv_decap_once(struct ipsec_rcv_state *irs)
 			return IPSEC_RCV_BADLEN;
 		}
 
-		if(proto_funcs->setup_auth) {
+		if(proto_funcs->rcv_setup_auth) {
 			enum ipsec_rcv_value retval
-			    = (*proto_funcs->setup_auth)(irs, skb,
+			    = (*proto_funcs->rcv_setup_auth)(irs, skb,
 							 &replay,
 							 &authenticator);
 			if(retval < 0) {
@@ -1133,10 +472,10 @@ ipsec_rcv_decap_once(struct ipsec_rcv_state *irs)
 			    irs->ipsp->ips_authalg);
 
 		/* calculate authenticator */
-		if(proto_funcs->calc_auth == NULL) {
+		if(proto_funcs->rcv_calc_auth == NULL) {
 			return IPSEC_RCV_BADAUTH;
 		}
-		(*proto_funcs->calc_auth)(irs, skb);
+		(*proto_funcs->rcv_calc_auth)(irs, skb);
 
 		if (memcmp(irs->hash, authenticator, irs->authlen)) {
 			irs->ipsp->ips_errs.ips_auth_errs += 1;
@@ -1191,9 +530,9 @@ ipsec_rcv_decap_once(struct ipsec_rcv_state *irs)
 		}
 	}
 
-	if(proto_funcs->decrypt) {
+	if(proto_funcs->rcv_decrypt) {
 		enum ipsec_rcv_value retval =
-		  (*proto_funcs->decrypt)(irs);
+		  (*proto_funcs->rcv_decrypt)(irs);
 
 		if(retval != IPSEC_RCV_OK) {
 			return retval;
@@ -1246,13 +585,12 @@ ipsec_rcv_decap_once(struct ipsec_rcv_state *irs)
 	if(sysctl_ipsec_inbound_policy_check) {
 		if(ipsnext) {
 			if(
-				ipp->protocol != IPPROTO_AH
-				&& ipp->protocol != IPPROTO_ESP
-#ifdef CONFIG_IPSEC_IPCOMP
+			        ipp->protocol != IPPROTO_ESP
+#ifdef CONFIG_KLIPS_IPCOMP
 				&& ipp->protocol != IPPROTO_COMP
 				&& (ipsnext->ips_said.proto != IPPROTO_COMP
 				    || ipsnext->ips_inext)
-#endif /* CONFIG_IPSEC_IPCOMP */
+#endif /* CONFIG_KLIPS_IPCOMP */
 				&& ipp->protocol != IPPROTO_IPIP
 				) {
 				KLIPS_PRINT(debug_rcv,
@@ -1276,7 +614,7 @@ ipsec_rcv_decap_once(struct ipsec_rcv_state *irs)
 		}
 	}
 
-#ifdef CONFIG_IPSEC_IPCOMP
+#ifdef CONFIG_KLIPS_IPCOMP
 	/* update ipcomp ratio counters, even if no ipcomp packet is present */
 	if (ipsnext
 	    && ipsnext->ips_said.proto == IPPROTO_COMP
@@ -1284,7 +622,7 @@ ipsec_rcv_decap_once(struct ipsec_rcv_state *irs)
 		ipsnext->ips_comp_ratio_cbytes += ntohs(ipp->tot_len);
 		ipsnext->ips_comp_ratio_dbytes += ntohs(ipp->tot_len);
 	}
-#endif /* CONFIG_IPSEC_IPCOMP */
+#endif /* CONFIG_KLIPS_IPCOMP */
 
 	irs->ipsp->ips_life.ipl_bytes.ipl_count += irs->len;
 	irs->ipsp->ips_life.ipl_bytes.ipl_last   = irs->len;
@@ -1296,7 +634,7 @@ ipsec_rcv_decap_once(struct ipsec_rcv_state *irs)
 	irs->ipsp->ips_life.ipl_packets.ipl_count += 1;
 
 #ifdef CONFIG_NETFILTER
-	if(proto == IPPROTO_ESP || proto == IPPROTO_AH) {
+	if(proto == IPPROTO_ESP) {
 		skb->nfmark = (skb->nfmark & (~(IPsecSAref2NFmark(IPSEC_SA_REF_MASK))))
 			| IPsecSAref2NFmark(IPsecSA2SAref(irs->ipsp));
 		KLIPS_PRINT(debug_rcv & DB_RX_PKTRX,
@@ -1318,25 +656,24 @@ ipsec_rcv(struct sk_buff *skb)
 #ifdef NET_21
 ipsec_rcv(struct sk_buff *skb, unsigned short xlen)
 #else /* NET_21 */
-ipsec_rcv(struct sk_buff *skb, struct device *dev, struct options *opt,
-		__u32 daddr_unused, unsigned short xlen, __u32 saddr,
+ipsec_rcv(struct sk_buff *skb
+	  , struct net_device *dev
+	  , struct options *opt
+	  , __u32 daddr_unused, unsigned short xlen, __u32 saddr,
 				   int redo, struct inet_protocol *protocol)
 #endif /* NET_21 */
 #endif /* PROTO_HANDLER_SINGLE_PARM */
 {
 #ifdef NET_21
-#ifdef CONFIG_IPSEC_DEBUG
-	struct device *dev = skb->dev;
-#endif /* CONFIG_IPSEC_DEBUG */
+#ifdef CONFIG_KLIPS_DEBUG
+	struct net_device *dev = skb->dev;
+#endif /* CONFIG_KLIPS_DEBUG */
 #endif /* NET_21 */
 	unsigned char protoc;
 	struct iphdr *ipp;
-#if defined(CONFIG_IPSEC_ESP) || defined(CONFIG_IPSEC_AH)
-#endif /* defined(CONFIG_IPSEC_ESP) || defined(CONFIG_IPSEC_AH) */
-
 	struct ipsec_sa *ipsp = NULL;
 	struct net_device_stats *stats = NULL;		/* This device's statistics */
-	struct device *ipsecdev = NULL, *prvdev;
+	struct net_device *ipsecdev = NULL, *prvdev;
 	struct ipsecpriv *prv;
 	char name[9];
 	int i;
@@ -1456,11 +793,7 @@ ipsec_rcv(struct sk_buff *skb, struct device *dev, struct options *opt,
 	}
 #endif /* !NET_21 */
 
-	if( (protoc != IPPROTO_AH) &&
-#ifdef CONFIG_IPSEC_IPCOMP_disabled_until_we_register_IPCOMP_HANDLER
-	    (protoc != IPPROTO_COMP) &&
-#endif /* CONFIG_IPSEC_IPCOMP */
-	    (protoc != IPPROTO_ESP) ) {
+	if( (protoc != IPPROTO_ESP) ) {
 		KLIPS_PRINT(debug_rcv & DB_RX_IPSA,
 			    "klips_debug:ipsec_rcv: Why the hell is someone "
 			    "passing me a non-ipsec protocol = %d packet? -- dropped.\n",
@@ -1489,7 +822,7 @@ ipsec_rcv(struct sk_buff *skb, struct device *dev, struct options *opt,
 					    name);
 			}
 			prv = ipsecdev ? (struct ipsecpriv *)(ipsecdev->priv) : NULL;
-			prvdev = prv ? (struct device *)(prv->dev) : NULL;
+			prvdev = prv ? (struct net_device *)(prv->dev) : NULL;
 
 #if 0
 			KLIPS_PRINT(debug_rcv && prvdev,
@@ -1554,8 +887,27 @@ ipsec_rcv(struct sk_buff *skb, struct device *dev, struct options *opt,
 
 	do {
 	        int decap_stat;
+		struct xform_functions *proto_funcs;
 
-	        decap_stat = ipsec_rcv_decap_once(&irs);
+		switch(irs.ipp->protocol) {
+		case IPPROTO_ESP:
+		  proto_funcs = esp_xform_funcs;
+		  break;
+		  
+#ifdef CONFIG_KLIPS_IPCOMP
+		case IPPROTO_COMP:
+		  proto_funcs = ipcomp_xform_funcs;
+		  break;
+#endif /* !CONFIG_KLIPS_IPCOMP */
+		default:
+		  if(irs.stats) {
+		    irs.stats->rx_errors++;
+		  }
+		  decap_stat = IPSEC_RCV_BADPROTO;
+		  goto rcvleave;
+		}
+
+	        decap_stat = ipsec_rcv_decap_once(&irs, proto_funcs);
 
 		if(decap_stat != IPSEC_RCV_OK) {
 			spin_unlock(&tdb_lock);
@@ -1567,10 +919,9 @@ ipsec_rcv(struct sk_buff *skb, struct device *dev, struct options *opt,
 		}
 	/* end decapsulation loop here */
 	} while(   (irs.ipp->protocol == IPPROTO_ESP )
-		|| (irs.ipp->protocol == IPPROTO_AH  )
-#ifdef CONFIG_IPSEC_IPCOMP
+#ifdef CONFIG_KLIPS_IPCOMP
 		|| (irs.ipp->protocol == IPPROTO_COMP)
-#endif /* CONFIG_IPSEC_IPCOMP */
+#endif /* CONFIG_KLIPS_IPCOMP */
 		);
 
 	/* set up for decap loop */
@@ -1582,12 +933,12 @@ ipsec_rcv(struct sk_buff *skb, struct device *dev, struct options *opt,
 	/* if there is an IPCOMP, but we don't have an IPPROTO_COMP,
 	 * then we can just skip it
 	 */
-#ifdef CONFIG_IPSEC_IPCOMP
+#ifdef CONFIG_KLIPS_IPCOMP
 	if(ipsnext && ipsnext->ips_said.proto == IPPROTO_COMP) {
 		ipsp = ipsnext;
 		ipsnext = ipsp->ips_inext;
 	}
-#endif /* CONFIG_IPSEC_IPCOMP */
+#endif /* CONFIG_KLIPS_IPCOMP */
 
 	/*
 	 * XXX this needs to be locked from when it was first looked
@@ -1596,7 +947,9 @@ ipsec_rcv(struct sk_buff *skb, struct device *dev, struct options *opt,
 	 */
 	if(ipsnext) {
 		ipsp = ipsnext;
-		irs.sa_len = satoa(irs.said, 0, irs.sa, SATOA_BUF);
+		irs.said = ipsnext->ips_said;
+		irs.sa_len = satot(&irs.said, 0, irs.sa, sizeof(irs.sa));
+
 		if(ipp->protocol != IPPROTO_IPIP) {
 			spin_unlock(&tdb_lock);
 			KLIPS_PRINT(debug_rcv,
@@ -1610,9 +963,9 @@ ipsec_rcv(struct sk_buff *skb, struct device *dev, struct options *opt,
 		}
 		if(sysctl_ipsec_inbound_policy_check) {
 			if((ipsnext = ipsp->ips_inext)) {
-				char sa2[SATOA_BUF];
+				char sa2[SATOT_BUF];
 				size_t sa_len2;
-				sa_len2 = satoa(ipsnext->ips_said, 0, sa2, SATOA_BUF);
+				sa_len2 = satot(&ipsnext->ips_said, 0, sa2, sizeof(sa2));
 				spin_unlock(&tdb_lock);
 				KLIPS_PRINT(debug_rcv,
 					    "klips_debug:ipsec_rcv: "
@@ -1750,7 +1103,7 @@ ipsec_rcv(struct sk_buff *skb, struct device *dev, struct options *opt,
 	}
 #endif /* NET_21 */
 
-#ifdef CONFIG_IPSEC_IPCOMP
+#ifdef CONFIG_KLIPS_IPCOMP
 	if(ipp->protocol == IPPROTO_COMP) {
 		unsigned int flags = 0;
 
@@ -1779,7 +1132,7 @@ ipsec_rcv(struct sk_buff *skb, struct device *dev, struct options *opt,
 			goto rcvleave;
 		}
 	}
-#endif /* CONFIG_IPSEC_IPCOMP */
+#endif /* CONFIG_KLIPS_IPCOMP */
 
 #ifdef SKB_RESET_NFCT
 	nf_conntrack_put(skb->nfct);
@@ -1805,60 +1158,41 @@ ipsec_rcv(struct sk_buff *skb, struct device *dev, struct options *opt,
 	return(0);
 }
 
-struct inet_protocol ah_protocol =
-{
-	ipsec_rcv,				/* AH handler */
-	NULL,				/* TUNNEL error control */
-#ifdef NETDEV_25
-	1,				/* no policy */
-#else
-	0,				/* next */
-	IPPROTO_AH,			/* protocol ID */
-	0,				/* copy */
-	NULL,				/* data */
-	"AH"				/* name */
-#endif
-};
-
-struct inet_protocol esp_protocol =
-{
-	ipsec_rcv,			/* ESP handler		*/
-	NULL,				/* TUNNEL error control */
-#ifdef NETDEV_25
-	1,				/* no policy */
-#else
-	0,				/* next */
-	IPPROTO_ESP,			/* protocol ID */
-	0,				/* copy */
-	NULL,				/* data */
-	"ESP"				/* name */
-#endif
-};
-
-#if 0
-/* We probably don't want to install a pure IPCOMP protocol handler, but
-   only want to handle IPCOMP if it is encapsulated inside an ESP payload
-   (which is already handled) */
-#ifdef CONFIG_IPSEC_IPCOMP
-struct inet_protocol comp_protocol =
-{
-	ipsec_rcv,			/* COMP handler		*/
-	NULL,				/* COMP error control	*/
-#ifdef NETDEV_25
-	1,				/* no policy */
-#else
-	0,				/* next */
-	IPPROTO_COMP,			/* protocol ID */
-	0,				/* copy */
-	NULL,				/* data */
-	"COMP"				/* name */
-#endif
-};
-#endif /* CONFIG_IPSEC_IPCOMP */
-#endif
-
 /*
  * $Log: ipsec_rcv.c,v $
+ * Revision 1.141  2004/02/24 17:17:04  mcr
+ * 	s/CONFIG_IPSEC/CONFIG_KLIPS/ as 26sec uses "CONFIG_IPSEC" to
+ * 	turn it on/off as well.
+ *
+ * Revision 1.140  2004/02/22 06:50:42  mcr
+ * 	kernel 2.6 port - merged with 2.4 code.
+ *
+ * Revision 1.139.6.1  2004/02/20 14:10:18  mcr
+ * 	moved code to net/ipsec/ to make 2.6 happy.
+ *
+ * Revision 1.139  2004/01/06 05:35:17  mcr
+ * 	fixes to initialize sin_family field for satot().
+ *
+ * Revision 1.138  2003/12/13 04:09:21  mcr
+ * 	AH transform removed.
+ *
+ * Revision 1.137  2003/12/11 20:15:04  mcr
+ * 	refactored the xmit code, to move all encapsulation
+ * 	code into protocol functions. Note that all functions
+ * 	are essentially done by a single function, which is probably
+ * 	wrong.
+ * 	the rcv_functions structures are renamed xform_functions.
+ *
+ * Revision 1.136  2003/12/06 21:21:38  mcr
+ * 	split up receive path into per-transform files, for
+ * 	easier later removal.
+ *
+ * Revision 1.135  2003/12/04 19:05:54  mcr
+ * 	cleaned up "sa_id" structure to use "ip_said" only.
+ *
+ * Revision 1.134  2003/11/07 02:58:06  mcr
+ * 	backout of port-selector and X.509 patches
+ *
  * Revision 1.132  2003/09/02 19:51:48  mcr
  * 	fixes for PR#252.
  *
