@@ -14,7 +14,7 @@
  * for more details.
  */
 
-char ipsec_xmit_c_version[] = "RCSID $Id: ipsec_xmit.c,v 1.4 2003/06/20 02:28:10 mcr Exp $";
+char ipsec_xmit_c_version[] = "RCSID $Id: ipsec_xmit.c,v 1.14 2004/02/24 17:17:04 mcr Exp $";
 
 #define __NO_VERSION__
 #include <linux/module.h>
@@ -65,17 +65,18 @@ char ipsec_xmit_c_version[] = "RCSID $Id: ipsec_xmit.c,v 1.4 2003/06/20 02:28:10
 #include "freeswan/ipsec_eroute.h"
 #include "freeswan/ipsec_encap.h"
 #include "freeswan/ipsec_radij.h"
-#include "freeswan/ipsec_netlink.h"
 #include "freeswan/ipsec_xmit.h"
 #include "freeswan/ipsec_sa.h"
 #include "freeswan/ipsec_tunnel.h"
 #include "freeswan/ipsec_ipe4.h"
-#include "freeswan/ipsec_ah.h"
-#include "freeswan/ipsec_esp.h"
+#include "freeswan/ipsec_auth.h"
 
-#ifdef CONFIG_IPSEC_IPCOMP
-#include "freeswan/ipcomp.h"
-#endif /* CONFIG_IPSEC_IPCOMP */
+#include "freeswan/ipsec_esp.h"
+#include "freeswan/ipsec_ipip.h"
+
+#ifdef CONFIG_KLIPS_IPCOMP
+#include "freeswan/ipsec_ipcomp.h"
+#endif /* CONFIG_KLIPS_IPCOMP */
 
 #include <pfkeyv2.h>
 #include <pfkey.h>
@@ -83,40 +84,9 @@ char ipsec_xmit_c_version[] = "RCSID $Id: ipsec_xmit.c,v 1.4 2003/06/20 02:28:10
 #include "freeswan/ipsec_proto.h"
 
 
-/* 
- * Stupid kernel API differences in APIs. Not only do some
- * kernels not have ip_select_ident, but some have differing APIs,
- * and SuSE has one with one parameter, but no way of checking to
- * see what is really what.
- */
-
-#ifdef SUSE_LINUX_2_4_19_IS_STUPID
-#define KLIPS_IP_SELECT_IDENT(iph, skb) ip_select_ident(iph)
-#else
-
-/* simplest case, nothing */
-#if !defined(IP_SELECT_IDENT)
-#define KLIPS_IP_SELECT_IDENT(iph, skb)  do { iph->id = htons(ip_id_count++); } while(0)
-#endif
-
-/* kernels > 2.3.37-ish */
-#if defined(IP_SELECT_IDENT) && !defined(IP_SELECT_IDENT_NEW)
-#define KLIPS_IP_SELECT_IDENT(iph, skb) ip_select_ident(iph, skb->dst)
-#endif
-
-/* kernels > 2.4.2 */
-#if defined(IP_SELECT_IDENT) && defined(IP_SELECT_IDENT_NEW)
-#define KLIPS_IP_SELECT_IDENT(iph, skb) ip_select_ident(iph, skb->dst, NULL)
-#endif
-
-#endif /* SUSE_LINUX_2_4_19_IS_STUPID */
-
-
-static __u32 zeroes[64];
-
-#ifdef CONFIG_IPSEC_DEBUG
+#ifdef CONFIG_KLIPS_DEBUG
 int sysctl_ipsec_debug_verbose = 0;
-#endif /* CONFIG_IPSEC_DEBUG */
+#endif /* CONFIG_KLIPS_DEBUG */
 
 int ipsec_xmit_trap_count = 0;
 int ipsec_xmit_trap_sendcount = 0;
@@ -124,7 +94,7 @@ int ipsec_xmit_trap_sendcount = 0;
 int sysctl_ipsec_icmp = 0;
 int sysctl_ipsec_tos = 0;
 
-#ifdef CONFIG_IPSEC_DEBUG_
+#ifdef CONFIG_KLIPS_DEBUG_
 DEBUG_NO_STATIC void
 dmp(char *s, caddr_t bb, int len)
 {
@@ -145,9 +115,9 @@ dmp(char *s, caddr_t bb, int len)
 		printk("\n");
 	}
 }
-#else /* CONFIG_IPSEC_DEBUG */
+#else /* CONFIG_KLIPS_DEBUG */
 #define dmp(_x, _y, _z) 
-#endif /* CONFIG_IPSEC_DEBUG */
+#endif /* CONFIG_KLIPS_DEBUG */
 
 #ifndef SKB_COPY_EXPAND
 /*
@@ -268,7 +238,7 @@ skb_copy_expand(struct sk_buff *skb, int headroom, int tailroom, int priority)
 }
 #endif /* !SKB_COPY_EXPAND */
 
-#ifdef CONFIG_IPSEC_DEBUG
+#ifdef CONFIG_KLIPS_DEBUG
 void
 ipsec_print_ip(struct iphdr *ip)
 {
@@ -337,7 +307,7 @@ ipsec_print_ip(struct iphdr *ip)
 		}
 	}
 }
-#endif /* CONFIG_IPSEC_DEBUG */
+#endif /* CONFIG_KLIPS_DEBUG */
 
 #ifdef MSS_HACK
 /*
@@ -501,99 +471,71 @@ ipsec_xmit_sanity_check_skb(struct ipsec_xmit_state *ixs)
 }
 
 enum ipsec_xmit_value
-ipsec_xmit_encap_once(struct ipsec_xmit_state *ixs)
+ipsec_xmit_encap_once(struct ipsec_xmit_state *ixs,
+		      struct xform_functions *protofuncs)
 {
-#ifdef CONFIG_IPSEC_ESP
-	struct esphdr *espp;
-	__u32 iv[2];
-	unsigned char *idat, *pad;
-	int authlen = 0, padlen = 0, i;
-#endif /* !CONFIG_IPSEC_ESP */
-#ifdef CONFIG_IPSEC_AH
-	struct iphdr ipo;
-	struct ahhdr *ahp;
-#endif /* CONFIG_IPSEC_AH */
-#if defined(CONFIG_IPSEC_AUTH_HMAC_MD5) || defined(CONFIG_IPSEC_AUTH_HMAC_SHA1)
-	union {
-#ifdef CONFIG_IPSEC_AUTH_HMAC_MD5
-		MD5_CTX md5;
-#endif /* CONFIG_IPSEC_AUTH_HMAC_MD5 */
-#ifdef CONFIG_IPSEC_AUTH_HMAC_SHA1
-		SHA1_CTX sha1;
-#endif /* CONFIG_IPSEC_AUTH_HMAC_SHA1 */
-	} tctx;
-	__u8 hash[AH_AMAX];
-#endif /* defined(CONFIG_IPSEC_AUTH_HMAC_MD5) || defined(CONFIG_IPSEC_AUTH_HMAC_SHA1) */
 	int headroom = 0, tailroom = 0, ilen = 0, len = 0;
 	unsigned char *dat;
+#if 0
+	__u8 hash[AH_AMAX];
+	union {
+#ifdef CONFIG_KLIPS_AUTH_HMAC_MD5
+		MD5_CTX md5;
+#endif /* CONFIG_KLIPS_AUTH_HMAC_MD5 */
+#ifdef CONFIG_KLIPS_AUTH_HMAC_SHA1
+		SHA1_CTX sha1;
+#endif /* CONFIG_KLIPS_AUTH_HMAC_SHA1 */
+	} tctx;
+	int padlen = 0, i;
+	unsigned char *idat, *pad;
+	__u32 iv[2];
+	struct esphdr *espp;
+#endif
 	
 	ixs->iphlen = ixs->iph->ihl << 2;
 	ixs->pyldsz = ntohs(ixs->iph->tot_len) - ixs->iphlen;
-	ixs->sa_len = satoa(ixs->ipsp->ips_said, 0, ixs->sa_txt, SATOA_BUF);
+	ixs->sa_len = satot(&ixs->ipsp->ips_said, 0, ixs->sa_txt, SATOT_BUF);
 	KLIPS_PRINT(debug_tunnel & DB_TN_OXFS,
 		    "klips_debug:ipsec_xmit_encap_once: "
 		    "calling output for <%s%s%s>, SA:%s\n", 
 		    IPS_XFORM_NAME(ixs->ipsp),
 		    ixs->sa_len ? ixs->sa_txt : " (error)");
 	
-	switch(ixs->ipsp->ips_said.proto) {
-#ifdef CONFIG_IPSEC_AH
-	case IPPROTO_AH:
-		headroom += sizeof(struct ahhdr);
-		break;
-#endif /* CONFIG_IPSEC_AH */
-#ifdef CONFIG_IPSEC_ESP
-	case IPPROTO_ESP:
-		switch(ixs->ipsp->ips_encalg) {
-#ifdef CONFIG_IPSEC_ENC_3DES
-		case ESP_3DES:
-			headroom += sizeof(struct esphdr);
-			break;
-#endif /* CONFIG_IPSEC_ENC_3DES */
-		default:
-			ixs->stats->tx_errors++;
-			return IPSEC_XMIT_ESP_BADALG;
-		}
-		switch(ixs->ipsp->ips_authalg) {
-#ifdef CONFIG_IPSEC_AUTH_HMAC_MD5
-		case AH_MD5:
-			authlen = AHHMAC_HASHLEN;
-			break;
-#endif /* CONFIG_IPSEC_AUTH_HMAC_MD5 */
-#ifdef CONFIG_IPSEC_AUTH_HMAC_SHA1
-		case AH_SHA:
-			authlen = AHHMAC_HASHLEN;
-			break;
-#endif /* CONFIG_IPSEC_AUTH_HMAC_SHA1 */
-		case AH_NONE:
-			break;
-		default:
-			ixs->stats->tx_errors++;
-			return IPSEC_XMIT_ESP_BADALG;
-		}		
-		tailroom += ((8 - ((ixs->pyldsz + 2 * sizeof(unsigned char)) % 8)) % 8) + 2;
-		tailroom += authlen;
-		break;
-#endif /* !CONFIG_IPSEC_ESP */
-#ifdef CONFIG_IPSEC_IPIP
-	case IPPROTO_IPIP:
-		headroom += sizeof(struct iphdr);
-		ixs->iphlen = sizeof(struct iphdr);
-		break;
-#endif /* !CONFIG_IPSEC_IPIP */
-#ifdef CONFIG_IPSEC_IPCOMP
-	case IPPROTO_COMP:
-		break;
-#endif /* CONFIG_IPSEC_IPCOMP */
+	headroom += protofuncs->xmit_headroom;
+
+	switch(ixs->ipsp->ips_authalg) {
+#ifdef CONFIG_KLIPS_AUTH_HMAC_MD5
+	case AH_MD5:
+	  ixs->authlen = AHHMAC_HASHLEN;
+	  break;
+#endif /* CONFIG_KLIPS_AUTH_HMAC_MD5 */
+#ifdef CONFIG_KLIPS_AUTH_HMAC_SHA1
+	case AH_SHA:
+	  ixs->authlen = AHHMAC_HASHLEN;
+	  break;
+#endif /* CONFIG_KLIPS_AUTH_HMAC_SHA1 */
+	case AH_NONE:
+	  break;
 	default:
-		ixs->stats->tx_errors++;
-		return IPSEC_XMIT_BADPROTO;
+	  ixs->stats->tx_errors++;
+	  return IPSEC_XMIT_ESP_BADALG;
+	}		
+
+	if(protofuncs->xmit_needtailroom) {
+	  tailroom += ((8 - ((ixs->pyldsz + 2 * sizeof(unsigned char)) % 8)) % 8) + 2;
+	  tailroom += ixs->authlen;
 	}
-	
+
+	/*
+	case IPPROTO_IPIP:
+		ixs->iphlen = sizeof(struct iphdr);
+	*/
+
 	KLIPS_PRINT(debug_tunnel & DB_TN_CROUT,
 		    "klips_debug:ipsec_xmit_encap_once: "
 		    "pushing %d bytes, putting %d, proto %d.\n", 
 		    headroom, tailroom, ixs->ipsp->ips_said.proto);
+
 	if(skb_headroom(ixs->skb) < headroom) {
 		printk(KERN_WARNING
 		       "klips_error:ipsec_xmit_encap_once: "
@@ -609,15 +551,17 @@ ipsec_xmit_encap_once(struct ipsec_xmit_state *ixs)
 		       "klips_error:ipsec_xmit_encap_once: "
 		       "tried to skb_put %d, %d available.  This should never happen, please report.\n",
 		       tailroom, skb_tailroom(ixs->skb));
-		ixs->stats->tx_errors++;
+	ixs->stats->tx_errors++;
 		return IPSEC_XMIT_ESP_PUSHPULLERR;
 	}
+
 	skb_put(ixs->skb, tailroom);
 	KLIPS_PRINT(debug_tunnel & DB_TN_CROUT,
 		    "klips_debug:ipsec_xmit_encap_once: "
 		    "head,tailroom: %d,%d before xform.\n",
 		    skb_headroom(ixs->skb), skb_tailroom(ixs->skb));
 	len = ixs->skb->len;
+
 	if(len > 0xfff0) {
 		printk(KERN_WARNING "klips_error:ipsec_xmit_encap_once: "
 		       "tot_len (%d) > 65520.  This should never happen, please report.\n",
@@ -628,293 +572,16 @@ ipsec_xmit_encap_once(struct ipsec_xmit_state *ixs)
 	memmove((void *)dat, (void *)(dat + headroom), ixs->iphlen);
 	ixs->iph = (struct iphdr *)dat;
 	ixs->iph->tot_len = htons(ixs->skb->len);
-	
-	switch(ixs->ipsp->ips_said.proto) {
-#ifdef CONFIG_IPSEC_ESP
-	case IPPROTO_ESP:
-		espp = (struct esphdr *)(dat + ixs->iphlen);
-		espp->esp_spi = ixs->ipsp->ips_said.spi;
-		espp->esp_rpl = htonl(++(ixs->ipsp->ips_replaywin_lastseq));
-		
-		switch(ixs->ipsp->ips_encalg) {
-#if defined(CONFIG_IPSEC_ENC_3DES)
-#ifdef CONFIG_IPSEC_ENC_3DES
-		case ESP_3DES:
-#endif /* CONFIG_IPSEC_ENC_3DES */
-			iv[0] = *((__u32*)&(espp->esp_iv)    ) =
-				((__u32*)(ixs->ipsp->ips_iv))[0];
-			iv[1] = *((__u32*)&(espp->esp_iv) + 1) =
-				((__u32*)(ixs->ipsp->ips_iv))[1];
-			break;
-#endif /* defined(CONFIG_IPSEC_ENC_3DES) */
-		default:
-			ixs->stats->tx_errors++;
-			return IPSEC_XMIT_ESP_BADALG;
-		}
-		
-		idat = dat + ixs->iphlen + headroom;
-		ilen = len - (ixs->iphlen + headroom + authlen);
-		
-		/* Self-describing padding */
-		pad = &dat[len - tailroom];
-		padlen = tailroom - 2 - authlen;
-		for (i = 0; i < padlen; i++) {
-			pad[i] = i + 1; 
-		}
-		dat[len - authlen - 2] = padlen;
-		
-		dat[len - authlen - 1] = ixs->iph->protocol;
-		ixs->iph->protocol = IPPROTO_ESP;
-		
-		switch(ixs->ipsp->ips_encalg) {
-#ifdef CONFIG_IPSEC_ENC_3DES
-		case ESP_3DES:
-			des_ede3_cbc_encrypt((des_cblock *)idat,
-					     (des_cblock *)idat,
-					     ilen,
-					     ((struct des_eks *)(ixs->ipsp->ips_key_e))[0].ks,
-					     ((struct des_eks *)(ixs->ipsp->ips_key_e))[1].ks,
-					     ((struct des_eks *)(ixs->ipsp->ips_key_e))[2].ks,
-					     (des_cblock *)iv, 1);
-			break;
-#endif /* CONFIG_IPSEC_ENC_3DES */
-		default:
-			ixs->stats->tx_errors++;
-			return IPSEC_XMIT_ESP_BADALG;
-		}
-		
-		switch(ixs->ipsp->ips_encalg) {
-#if defined(CONFIG_IPSEC_ENC_3DES)
-#ifdef CONFIG_IPSEC_ENC_3DES
-		case ESP_3DES:
-#endif /* CONFIG_IPSEC_ENC_3DES */
-			/* XXX update IV with the last 8 octets of the encryption */
-#if KLIPS_IMPAIRMENT_ESPIV_CBC_ATTACK
-			((__u32*)(ixs->ipsp->ips_iv))[0] =
-				((__u32 *)(idat))[(ilen >> 2) - 2];
-			((__u32*)(ixs->ipsp->ips_iv))[1] =
-				((__u32 *)(idat))[(ilen >> 2) - 1];
-#else /* KLIPS_IMPAIRMENT_ESPIV_CBC_ATTACK */
-			prng_bytes(&ipsec_prng, (char *)ixs->ipsp->ips_iv, EMT_ESPDES_IV_SZ); 
-#endif /* KLIPS_IMPAIRMENT_ESPIV_CBC_ATTACK */
-			break;
-#endif /* defined(CONFIG_IPSEC_ENC_3DES) */
-		default:
-			ixs->stats->tx_errors++;
-			return IPSEC_XMIT_ESP_BADALG;
-		}
-		
-		switch(ixs->ipsp->ips_authalg) {
-#ifdef CONFIG_IPSEC_AUTH_HMAC_MD5
-		case AH_MD5:
-			dmp("espp", (char*)espp, len - ixs->iphlen - authlen);
-			tctx.md5 = ((struct md5_ctx*)(ixs->ipsp->ips_key_a))->ictx;
-			dmp("ictx", (char*)&tctx.md5, sizeof(tctx.md5));
-			MD5Update(&tctx.md5, (caddr_t)espp, len - ixs->iphlen - authlen);
-			dmp("ictx+dat", (char*)&tctx.md5, sizeof(tctx.md5));
-			MD5Final(hash, &tctx.md5);
-			dmp("ictx hash", (char*)&hash, sizeof(hash));
-			tctx.md5 = ((struct md5_ctx*)(ixs->ipsp->ips_key_a))->octx;
-			dmp("octx", (char*)&tctx.md5, sizeof(tctx.md5));
-			MD5Update(&tctx.md5, hash, AHMD596_ALEN);
-			dmp("octx+hash", (char*)&tctx.md5, sizeof(tctx.md5));
-			MD5Final(hash, &tctx.md5);
-			dmp("octx hash", (char*)&hash, sizeof(hash));
-			memcpy(&(dat[len - authlen]), hash, authlen);
-			
-			/* paranoid */
-			memset((caddr_t)&tctx.md5, 0, sizeof(tctx.md5));
-			memset((caddr_t)hash, 0, sizeof(*hash));
-			break;
-#endif /* CONFIG_IPSEC_AUTH_HMAC_MD5 */
-#ifdef CONFIG_IPSEC_AUTH_HMAC_SHA1
-		case AH_SHA:
-			tctx.sha1 = ((struct sha1_ctx*)(ixs->ipsp->ips_key_a))->ictx;
-			SHA1Update(&tctx.sha1, (caddr_t)espp, len - ixs->iphlen - authlen);
-			SHA1Final(hash, &tctx.sha1);
-			tctx.sha1 = ((struct sha1_ctx*)(ixs->ipsp->ips_key_a))->octx;
-			SHA1Update(&tctx.sha1, hash, AHSHA196_ALEN);
-			SHA1Final(hash, &tctx.sha1);
-			memcpy(&(dat[len - authlen]), hash, authlen);
-			
-			/* paranoid */
-			memset((caddr_t)&tctx.sha1, 0, sizeof(tctx.sha1));
-			memset((caddr_t)hash, 0, sizeof(*hash));
-			break;
-#endif /* CONFIG_IPSEC_AUTH_HMAC_SHA1 */
-		case AH_NONE:
-			break;
-		default:
-			ixs->stats->tx_errors++;
-			return IPSEC_XMIT_AH_BADALG;
-		}
-#ifdef NET_21
-		ixs->skb->h.raw = (unsigned char*)espp;
-#endif /* NET_21 */
-		break;
-#endif /* !CONFIG_IPSEC_ESP */
-#ifdef CONFIG_IPSEC_AH
-	case IPPROTO_AH:
-		ahp = (struct ahhdr *)(dat + ixs->iphlen);
-		ahp->ah_spi = ixs->ipsp->ips_said.spi;
-		ahp->ah_rpl = htonl(++(ixs->ipsp->ips_replaywin_lastseq));
-		ahp->ah_rv = 0;
-		ahp->ah_nh = ixs->iph->protocol;
-		ahp->ah_hl = (headroom >> 2) - sizeof(__u64)/sizeof(__u32);
-		ixs->iph->protocol = IPPROTO_AH;
-		dmp("ahp", (char*)ahp, sizeof(*ahp));
-		
-		ipo = *ixs->iph;
-		ipo.tos = 0;
-		ipo.frag_off = 0;
-		ipo.ttl = 0;
-		ipo.check = 0;
-		dmp("ipo", (char*)&ipo, sizeof(ipo));
-		
-		switch(ixs->ipsp->ips_authalg) {
-#ifdef CONFIG_IPSEC_AUTH_HMAC_MD5
-		case AH_MD5:
-			tctx.md5 = ((struct md5_ctx*)(ixs->ipsp->ips_key_a))->ictx;
-			dmp("ictx", (char*)&tctx.md5, sizeof(tctx.md5));
-			MD5Update(&tctx.md5, (unsigned char *)&ipo, sizeof (struct iphdr));
-			dmp("ictx+ipo", (char*)&tctx.md5, sizeof(tctx.md5));
-			MD5Update(&tctx.md5, (unsigned char *)ahp, headroom - sizeof(ahp->ah_data));
-			dmp("ictx+ahp", (char*)&tctx.md5, sizeof(tctx.md5));
-			MD5Update(&tctx.md5, (unsigned char *)zeroes, AHHMAC_HASHLEN);
-			dmp("ictx+zeroes", (char*)&tctx.md5, sizeof(tctx.md5));
-			MD5Update(&tctx.md5,  dat + ixs->iphlen + headroom, len - ixs->iphlen - headroom);
-			dmp("ictx+dat", (char*)&tctx.md5, sizeof(tctx.md5));
-			MD5Final(hash, &tctx.md5);
-			dmp("ictx hash", (char*)&hash, sizeof(hash));
-			tctx.md5 = ((struct md5_ctx*)(ixs->ipsp->ips_key_a))->octx;
-			dmp("octx", (char*)&tctx.md5, sizeof(tctx.md5));
-			MD5Update(&tctx.md5, hash, AHMD596_ALEN);
-			dmp("octx+hash", (char*)&tctx.md5, sizeof(tctx.md5));
-			MD5Final(hash, &tctx.md5);
-			dmp("octx hash", (char*)&hash, sizeof(hash));
-					
-			memcpy(ahp->ah_data, hash, AHHMAC_HASHLEN);
-					
-			/* paranoid */
-			memset((caddr_t)&tctx.md5, 0, sizeof(tctx.md5));
-			memset((caddr_t)hash, 0, sizeof(*hash));
-			break;
-#endif /* CONFIG_IPSEC_AUTH_HMAC_MD5 */
-#ifdef CONFIG_IPSEC_AUTH_HMAC_SHA1
-		case AH_SHA:
-			tctx.sha1 = ((struct sha1_ctx*)(ixs->ipsp->ips_key_a))->ictx;
-			SHA1Update(&tctx.sha1, (unsigned char *)&ipo, sizeof (struct iphdr));
-			SHA1Update(&tctx.sha1, (unsigned char *)ahp, headroom - sizeof(ahp->ah_data));
-			SHA1Update(&tctx.sha1, (unsigned char *)zeroes, AHHMAC_HASHLEN);
-			SHA1Update(&tctx.sha1,  dat + ixs->iphlen + headroom, len - ixs->iphlen - headroom);
-			SHA1Final(hash, &tctx.sha1);
-			tctx.sha1 = ((struct sha1_ctx*)(ixs->ipsp->ips_key_a))->octx;
-			SHA1Update(&tctx.sha1, hash, AHSHA196_ALEN);
-			SHA1Final(hash, &tctx.sha1);
-					
-			memcpy(ahp->ah_data, hash, AHHMAC_HASHLEN);
-					
-			/* paranoid */
-			memset((caddr_t)&tctx.sha1, 0, sizeof(tctx.sha1));
-			memset((caddr_t)hash, 0, sizeof(*hash));
-			break;
-#endif /* CONFIG_IPSEC_AUTH_HMAC_SHA1 */
-		default:
-			ixs->stats->tx_errors++;
-			return IPSEC_XMIT_AH_BADALG;
-		}
-#ifdef NET_21
-		ixs->skb->h.raw = (unsigned char*)ahp;
-#endif /* NET_21 */
-		break;
-#endif /* CONFIG_IPSEC_AH */
-#ifdef CONFIG_IPSEC_IPIP
-	case IPPROTO_IPIP:
-		ixs->iph->version  = 4;
-		switch(sysctl_ipsec_tos) {
-		case 0:
-#ifdef NET_21
-			ixs->iph->tos = ixs->skb->nh.iph->tos;
-#else /* NET_21 */
-			ixs->iph->tos = ixs->skb->ip_hdr->tos;
-#endif /* NET_21 */
-			break;
-		case 1:
-			ixs->iph->tos = 0;
-			break;
-		default:
-			break;
-		}
-#ifdef NET_21
-#ifdef NETDEV_23
-		ixs->iph->ttl      = sysctl_ip_default_ttl;
-#else /* NETDEV_23 */
-		ixs->iph->ttl      = ip_statistics.IpDefaultTTL;
-#endif /* NETDEV_23 */
-#else /* NET_21 */
-		ixs->iph->ttl      = 64; /* ip_statistics.IpDefaultTTL; */
-#endif /* NET_21 */
-		ixs->iph->frag_off = 0;
-		ixs->iph->saddr    = ((struct sockaddr_in*)(ixs->ipsp->ips_addr_s))->sin_addr.s_addr;
-		ixs->iph->daddr    = ((struct sockaddr_in*)(ixs->ipsp->ips_addr_d))->sin_addr.s_addr;
-		ixs->iph->protocol = IPPROTO_IPIP;
-		ixs->iph->ihl      = sizeof(struct iphdr) >> 2;
+	ixs->tailroom = tailroom;
 
-		KLIPS_IP_SELECT_IDENT(ixs->iph, ixs->skb);
-
-		ixs->newdst = (__u32)ixs->iph->daddr;
-		ixs->newsrc = (__u32)ixs->iph->saddr;
-		
-#ifdef NET_21
-		ixs->skb->h.ipiph = ixs->skb->nh.iph;
-#endif /* NET_21 */
-		break;
-#endif /* !CONFIG_IPSEC_IPIP */
-#ifdef CONFIG_IPSEC_IPCOMP
-	case IPPROTO_COMP:
-	{
-		unsigned int flags = 0;
-#ifdef CONFIG_IPSEC_DEBUG
-		unsigned int old_tot_len = ntohs(ixs->iph->tot_len);
-#endif /* CONFIG_IPSEC_DEBUG */
-		ixs->ipsp->ips_comp_ratio_dbytes += ntohs(ixs->iph->tot_len);
-
-		ixs->skb = skb_compress(ixs->skb, ixs->ipsp, &flags);
-
-#ifdef NET_21
-		ixs->iph = ixs->skb->nh.iph;
-#else /* NET_21 */
-		ixs->iph = ixs->skb->ip_hdr;
-#endif /* NET_21 */
-
-		ixs->ipsp->ips_comp_ratio_cbytes += ntohs(ixs->iph->tot_len);
-
-#ifdef CONFIG_IPSEC_DEBUG
-		if (debug_tunnel & DB_TN_CROUT)
-		{
-			if (old_tot_len > ntohs(ixs->iph->tot_len))
-				KLIPS_PRINT(debug_tunnel & DB_TN_CROUT,
-					    "klips_debug:ipsec_xmit_encap_once: "
-					    "packet shrunk from %d to %d bytes after compression, cpi=%04x (should be from spi=%08x, spi&0xffff=%04x.\n",
-					    old_tot_len, ntohs(ixs->iph->tot_len),
-					    ntohs(((struct ipcomphdr*)(((char*)ixs->iph) + ((ixs->iph->ihl) << 2)))->ipcomp_cpi),
-					    ntohl(ixs->ipsp->ips_said.spi),
-					    (__u16)(ntohl(ixs->ipsp->ips_said.spi) & 0x0000ffff));
-			else
-				KLIPS_PRINT(debug_tunnel & DB_TN_CROUT,
-					    "klips_debug:ipsec_xmit_encap_once: "
-					    "packet did not compress (flags = %d).\n",
-					    flags);
-		}
-#endif /* CONFIG_IPSEC_DEBUG */
+	if(protofuncs->xmit_setup) {
+	  enum ipsec_xmit_value retval =
+	    (*protofuncs->xmit_setup)(ixs);
+	  if(retval < 0) {
+	    return retval;
+	  }
 	}
-	break;
-#endif /* CONFIG_IPSEC_IPCOMP */
-	default:
-		ixs->stats->tx_errors++;
-		return IPSEC_XMIT_BADPROTO;
-	}
-			
+
 #ifdef NET_21
 	ixs->skb->nh.raw = ixs->skb->data;
 #else /* NET_21 */
@@ -944,6 +611,247 @@ ipsec_xmit_encap_once(struct ipsec_xmit_state *ixs)
 	return IPSEC_XMIT_OK;
 }
 
+/*
+ * A TRAP eroute is installed and we want to replace it with a HOLD
+ * eroute.
+ */
+static int create_hold_eroute(struct eroute *origtrap,
+			      struct sk_buff * skb, struct iphdr * iph,
+			      uint32_t eroute_pid)
+{
+	struct eroute hold_eroute;
+	ip_said hold_said;
+	struct sk_buff *first, *last;
+	int error;
+
+	first = last = NULL;
+	memset((caddr_t)&hold_eroute, 0, sizeof(hold_eroute));
+	memset((caddr_t)&hold_said, 0, sizeof(hold_said));
+	
+	hold_said.proto = IPPROTO_INT;
+	hold_said.spi = htonl(SPI_HOLD);
+	hold_said.dst.u.v4.sin_addr.s_addr = INADDR_ANY;
+	
+	hold_eroute.er_eaddr.sen_len = sizeof(struct sockaddr_encap);
+	hold_eroute.er_emask.sen_len = sizeof(struct sockaddr_encap);
+	hold_eroute.er_eaddr.sen_family = AF_ENCAP;
+	hold_eroute.er_emask.sen_family = AF_ENCAP;
+	hold_eroute.er_eaddr.sen_type = SENT_IP4;
+	hold_eroute.er_emask.sen_type = 255;
+	
+	hold_eroute.er_eaddr.sen_ip_src.s_addr = iph->saddr;
+	hold_eroute.er_eaddr.sen_ip_dst.s_addr = iph->daddr;
+	hold_eroute.er_emask.sen_ip_src.s_addr = INADDR_BROADCAST;
+	hold_eroute.er_emask.sen_ip_dst.s_addr = INADDR_BROADCAST;
+	hold_eroute.er_pid = eroute_pid;
+	hold_eroute.er_count = 0;
+	hold_eroute.er_lasttime = jiffies/HZ;
+
+#ifdef CONFIG_KLIPS_DEBUG
+	if (debug_pfkey) {
+		char buf1[64], buf2[64];
+		subnettoa(hold_eroute.er_eaddr.sen_ip_src,
+			  hold_eroute.er_emask.sen_ip_src, 0, buf1, sizeof(buf1));
+		subnettoa(hold_eroute.er_eaddr.sen_ip_dst,
+			  hold_eroute.er_emask.sen_ip_dst, 0, buf2, sizeof(buf2));
+		KLIPS_PRINT(debug_pfkey,
+			    "klips_debug:ipsec_tunnel_start_xmit: "
+			    "calling breakeroute and makeroute for %s->%s HOLD eroute.\n",
+			    buf1, 
+			    buf2);
+	}
+#endif /* CONFIG_KLIPS_DEBUG */
+
+	if (ipsec_breakroute(&(hold_eroute.er_eaddr), &(hold_eroute.er_emask),
+			     &first, &last)) {
+		KLIPS_PRINT(debug_pfkey,
+			    "klips_debug:ipsec_tunnel_start_xmit: "
+			    "HOLD breakeroute found nothing.\n");
+	} else {
+		KLIPS_PRINT(debug_pfkey,
+			    "klips_debug:ipsec_tunnel_start_xmit: "
+			    "HOLD breakroute deleted %u.%u.%u.%u -> %u.%u.%u.%u\n",
+			    NIPQUAD(hold_eroute.er_eaddr.sen_ip_src),
+			    NIPQUAD(hold_eroute.er_eaddr.sen_ip_dst));
+	}
+	if (first != NULL)
+		kfree_skb(first);
+	if (last != NULL)
+		kfree_skb(last);
+
+	error = ipsec_makeroute(&(hold_eroute.er_eaddr),
+				&(hold_eroute.er_emask),
+				hold_said, eroute_pid, skb, NULL, NULL);
+	if (error) {
+		KLIPS_PRINT(debug_pfkey,
+			    "klips_debug:ipsec_tunnel_start_xmit: "
+			    "HOLD makeroute returned %d, failed.\n", error);
+	} else {
+		KLIPS_PRINT(debug_pfkey,
+			    "klips_debug:ipsec_tunnel_start_xmit: "
+			    "HOLD makeroute call successful.\n");
+	}
+	return (error == 0);
+}
+
+enum ipsec_xmit_value
+ipsec_xmit_encap_internal(struct ipsec_xmit_state *ixs)
+{
+	switch (ntohl(ixs->outgoing_said.spi)) {
+	case SPI_DROP:
+		KLIPS_PRINT(debug_tunnel & DB_TN_XMIT,
+			    "klips_debug:ipsec_xmit_encap_bundle: "
+			    "shunt SA of DROP or no eroute: dropping.\n");
+		ixs->stats->tx_dropped++;
+		break;
+		
+	case SPI_REJECT:
+		KLIPS_PRINT(debug_tunnel & DB_TN_XMIT,
+			    "klips_debug:ipsec_xmit_encap_bundle: "
+			    "shunt SA of REJECT: notifying and dropping.\n");
+		ICMP_SEND(ixs->skb,
+			  ICMP_DEST_UNREACH,
+			  ICMP_PKT_FILTERED,
+			  0,
+			  ixs->physdev);
+		ixs->stats->tx_dropped++;
+		break;
+		
+	case SPI_PASS:
+#ifdef NET_21
+		ixs->pass = 1;
+#endif /* NET_21 */
+		KLIPS_PRINT(debug_tunnel & DB_TN_XMIT,
+			    "klips_debug:ipsec_xmit_encap_bundle: "
+			    "PASS: calling dev_queue_xmit\n");
+		return IPSEC_XMIT_PASS;
+		
+	case SPI_HOLD:
+		KLIPS_PRINT(debug_tunnel & DB_TN_XMIT,
+			    "klips_debug:ipsec_xmit_encap_bundle: "
+			    "shunt SA of HOLD: this does not make sense here, dropping.\n");
+		ixs->stats->tx_dropped++;
+		break;
+		
+	case SPI_PASSTRAP:			
+	case SPI_TRAP:
+	case SPI_TRAPSUBNET:
+	{
+		struct sockaddr_in src, dst;
+		
+		/* Signal all listening KMds with a PF_KEY ACQUIRE */
+		
+		src.sin_family = AF_INET;
+		dst.sin_family = AF_INET;
+		src.sin_addr.s_addr = ixs->iph->saddr;
+		dst.sin_addr.s_addr = ixs->iph->daddr;
+		
+		ixs->ips.ips_transport_protocol = 0;
+		src.sin_port = 0;
+		dst.sin_port = 0;
+		{
+			int i;
+			for(i = 0;
+			    i < sizeof(struct sockaddr_in)
+				    - offsetof(struct sockaddr_in, sin_zero);
+			    i++) {
+				src.sin_zero[i] = 0;
+				dst.sin_zero[i] = 0;
+			}
+		}
+		
+		ixs->ips.ips_addr_s = (struct sockaddr*)(&src);
+		ixs->ips.ips_addr_d = (struct sockaddr*)(&dst);
+
+		if(KLIPS_PRINTIF(debug_tunnel & DB_TN_XMIT))
+		{
+			char bufsrc[ADDRTOA_BUF], bufdst[ADDRTOA_BUF];
+			struct sockaddr_in *s, *d;
+			char *bs, *bd;
+
+			s = (struct sockaddr_in*)(ixs->ips.ips_addr_s);
+			d = (struct sockaddr_in*)(ixs->ips.ips_addr_d);
+
+			bs = (addrtoa(s->sin_addr, 0,
+				      bufsrc, sizeof(bufsrc)) <= ADDRTOA_BUF)
+				? bufsrc : "BAD_ADDR";
+
+			bd = (addrtoa(d->sin_addr, 0,
+				      bufdst, sizeof(bufdst)) <= ADDRTOA_BUF)
+				? bufdst : "BAD_ADDR";
+
+			KLIPS_WRITE("klips_debug:ipsec_xmit_encap_bundle: "
+				    "SADB_ACQUIRE sent due to %d with "
+				    "src=%s:%d, dst=%s:%d, proto=%d.\n",
+				    ntohl(ixs->outgoing_said.spi),
+				    bs,
+				    ntohs(s->sin_port),
+				    bd, 
+				    ntohs(d->sin_port),
+				    ixs->ips.ips_said.proto);
+		}
+		
+		/* increment count of total traps needed */
+		ipsec_xmit_trap_count++;
+		
+		if (pfkey_acquire(&ixs->ips) == 0) {
+			
+			/* note that we succeeded */
+			ipsec_xmit_trap_sendcount++;
+			
+			if (ixs->outgoing_said.spi==htonl(SPI_TRAPSUBNET))
+			{
+				/*
+				 * The spinlock is to prevent any other
+				 * process from accessing or deleting
+				 * the eroute while we are using and
+				 * updating it.
+				 */
+				spin_lock(&eroute_lock);
+				ixs->eroute = ipsec_findroute(&ixs->matcher);
+				if(ixs->eroute) {
+					ixs->eroute->er_said.spi = htonl(SPI_HOLD);
+					ixs->eroute->er_first = ixs->skb;
+					ixs->skb = NULL;
+				}
+				spin_unlock(&eroute_lock);
+			}
+			else if (create_hold_eroute(ixs->eroute,
+						    ixs->skb,
+						    ixs->iph,
+						    ixs->eroute_pid))
+			{
+				if(ixs->outgoing_said.spi == htonl(SPI_PASSTRAP))
+				{
+					ixs->skb = skb_clone(ixs->skb, GFP_ATOMIC);
+				}
+				else
+				{
+					ixs->skb = NULL;
+				}
+			} 
+		}
+		/* whether or not the above succeeded, we continue */
+		
+		/* become a %pass if we were told to! */
+		if(ixs->outgoing_said.spi == htonl(SPI_PASSTRAP)) {
+			ixs->pass = 1;
+			KLIPS_PRINT(debug_tunnel & DB_TN_XMIT,
+				    "klips_debug:ipsec_xmit_encap_bundle: "
+				    "PASSTRAP: letting packet out after %%trap\n");
+			return IPSEC_XMIT_PASS;
+		}
+		
+		ixs->stats->tx_dropped++;
+	}
+	default:
+		/* XXX what do we do with an unknown shunt spi? */
+		break;
+	} /* end of switch */
+	
+	return IPSEC_XMIT_STOLEN;
+}
+
 enum ipsec_xmit_value
 ipsec_xmit_encap_bundle(struct ipsec_xmit_state *ixs)
 {
@@ -951,234 +859,14 @@ ipsec_xmit_encap_bundle(struct ipsec_xmit_state *ixs)
  
 	ixs->newdst = ixs->orgdst = ixs->iph->daddr;
 	ixs->newsrc = ixs->orgsrc = ixs->iph->saddr;
-	ixs->orgedst = ixs->outgoing_said.dst.s_addr;
+	ixs->orgedst = ixs->outgoing_said.dst.u.v4.sin_addr.s_addr;
 	ixs->iphlen = ixs->iph->ihl << 2;
 	ixs->pyldsz = ntohs(ixs->iph->tot_len) - ixs->iphlen;
 	ixs->max_headroom = ixs->max_tailroom = 0;
 		
 	if (ixs->outgoing_said.proto == IPPROTO_INT) {
-		switch (ntohl(ixs->outgoing_said.spi)) {
-		case SPI_DROP:
-			KLIPS_PRINT(debug_tunnel & DB_TN_XMIT,
-				    "klips_debug:ipsec_xmit_encap_bundle: "
-				    "shunt SA of DROP or no eroute: dropping.\n");
-			ixs->stats->tx_dropped++;
-			break;
-				
-		case SPI_REJECT:
-			KLIPS_PRINT(debug_tunnel & DB_TN_XMIT,
-				    "klips_debug:ipsec_xmit_encap_bundle: "
-				    "shunt SA of REJECT: notifying and dropping.\n");
-			ICMP_SEND(ixs->skb,
-				  ICMP_DEST_UNREACH,
-				  ICMP_PKT_FILTERED,
-				  0,
-				  ixs->physdev);
-			ixs->stats->tx_dropped++;
-			break;
-				
-		case SPI_PASS:
-#ifdef NET_21
-			ixs->pass = 1;
-#endif /* NET_21 */
-			KLIPS_PRINT(debug_tunnel & DB_TN_XMIT,
-				    "klips_debug:ipsec_xmit_encap_bundle: "
-				    "PASS: calling dev_queue_xmit\n");
-			return IPSEC_XMIT_PASS;
-			goto cleanup;
-				
-#if 1 /* now moved up to finderoute so we don't need to lock it longer */
-		case SPI_HOLD:
-			KLIPS_PRINT(debug_tunnel & DB_TN_XMIT,
-				    "klips_debug:ipsec_xmit_encap_bundle: "
-				    "shunt SA of HOLD: this does not make sense here, dropping.\n");
-			ixs->stats->tx_dropped++;
-			break;
-#endif		
-		case SPI_TRAP:
-		case SPI_TRAPSUBNET:
-		{
-			struct sockaddr_in src, dst;
-#ifdef CONFIG_IPSEC_DEBUG
-			char bufsrc[ADDRTOA_BUF], bufdst[ADDRTOA_BUF];
-#endif /* CONFIG_IPSEC_DEBUG */
-			struct eroute hold_eroute;
-			struct sa_id hold_said;
-			struct sk_buff *first = NULL, *last = NULL;
-				
-			/* Signal all listening KMds with a PF_KEY ACQUIRE */
-			ixs->ips.ips_said.proto = ixs->iph->protocol;
-			src.sin_family = AF_INET;
-			dst.sin_family = AF_INET;
-			src.sin_addr.s_addr = ixs->iph->saddr;
-			dst.sin_addr.s_addr = ixs->iph->daddr;
-			src.sin_port = 
-				(ixs->iph->protocol == IPPROTO_UDP
-				 ? ((struct udphdr*) (((caddr_t)ixs->iph) + (ixs->iph->ihl << 2)))->source
-				 : (ixs->iph->protocol == IPPROTO_TCP
-				    ? ((struct tcphdr*)((caddr_t)ixs->iph + (ixs->iph->ihl << 2)))->source
-				    : 0));
-			dst.sin_port = 
-				(ixs->iph->protocol == IPPROTO_UDP
-				 ? ((struct udphdr*) (((caddr_t)ixs->iph) + (ixs->iph->ihl << 2)))->dest
-				 : (ixs->iph->protocol == IPPROTO_TCP
-				    ? ((struct tcphdr*)((caddr_t)ixs->iph + (ixs->iph->ihl << 2)))->dest
-				    : 0));
-			{
-				int i;
-				for(i = 0;
-				    i < sizeof(struct sockaddr_in)
-					    - offsetof(struct sockaddr_in, sin_zero);
-				    i++) {
-					src.sin_zero[i] = 0;
-					dst.sin_zero[i] = 0;
-				}
-			}
-				
-			ixs->ips.ips_addr_s = (struct sockaddr*)(&src);
-			ixs->ips.ips_addr_d = (struct sockaddr*)(&dst);
-			KLIPS_PRINT(debug_tunnel & DB_TN_XMIT,
-				    "klips_debug:ipsec_xmit_encap_bundle: "
-				    "SADB_ACQUIRE sent with src=%s:%d, dst=%s:%d, proto=%d.\n",
-				    addrtoa(((struct sockaddr_in*)(ixs->ips.ips_addr_s))->sin_addr, 0, bufsrc, sizeof(bufsrc)) <= ADDRTOA_BUF ? bufsrc : "BAD_ADDR",
-				    ntohs(((struct sockaddr_in*)(ixs->ips.ips_addr_s))->sin_port),
-				    addrtoa(((struct sockaddr_in*)(ixs->ips.ips_addr_d))->sin_addr, 0, bufdst, sizeof(bufdst)) <= ADDRTOA_BUF ? bufdst : "BAD_ADDR",
-				    ntohs(((struct sockaddr_in*)(ixs->ips.ips_addr_d))->sin_port),
-				    ixs->ips.ips_said.proto);
-				
-			/* increment count of total traps needed */
-			ipsec_xmit_trap_count++;
-
-			if (pfkey_acquire(&ixs->ips) == 0) {
-
-				/* note that we succeeded */
-			        ipsec_xmit_trap_sendcount++;
-					
-				if (ixs->outgoing_said.spi==htonl(SPI_TRAPSUBNET)) {
-					/*
-					 * The spinlock is to prevent any other
-					 * process from accessing or deleting
-					 * the eroute while we are using and
-					 * updating it.
-					 */
-					spin_lock(&eroute_lock);
-					ixs->eroute = ipsec_findroute(&ixs->matcher);
-					if(ixs->eroute) {
-						ixs->eroute->er_said.spi = htonl(SPI_HOLD);
-						ixs->eroute->er_first = ixs->skb;
-						ixs->skb = NULL;
-					}
-					spin_unlock(&eroute_lock);
-				} else {
-					/* install HOLD eroute */
-					memset((caddr_t)&hold_eroute, 0, sizeof(hold_eroute));
-					memset((caddr_t)&hold_said, 0, sizeof(hold_said));
-						
-					hold_said.proto = IPPROTO_INT;
-					hold_said.spi = htonl(SPI_HOLD);
-					hold_said.dst.s_addr = INADDR_ANY;
-						
-					hold_eroute.er_eaddr.sen_len = sizeof(struct sockaddr_encap);
-					hold_eroute.er_emask.sen_len = sizeof(struct sockaddr_encap);
-					hold_eroute.er_eaddr.sen_family = AF_ENCAP;
-					hold_eroute.er_emask.sen_family = AF_ENCAP;
-					hold_eroute.er_eaddr.sen_type = SENT_IP4;
-					hold_eroute.er_emask.sen_type = 255;
-						
-					hold_eroute.er_eaddr.sen_ip_src.s_addr = ixs->iph->saddr;
-					hold_eroute.er_eaddr.sen_ip_dst.s_addr = ixs->iph->daddr;
-					hold_eroute.er_emask.sen_ip_src.s_addr = INADDR_BROADCAST;
-					hold_eroute.er_emask.sen_ip_dst.s_addr = INADDR_BROADCAST;
-						
-					hold_eroute.er_pid = ixs->eroute_pid;
-					hold_eroute.er_count = 0;
-					hold_eroute.er_lasttime = jiffies/HZ;
-						
-#ifdef IPSEC_CONFIG_FULL_SELECTOR_LIST
-					/* These are ficticious.  Don't uncomment these until
-					   proto, sport and dport exist in the SPDB */
-					hold_eroute.er_proto = ixs->iph->protocol;
-					hold_eroute.er_src_port = 
-						(ixs->iph->protocol == IPPROTO_UDP
-						 ? ((struct udphdr*) (((caddr_t)ixs->iph) + (ixs->iph->ihl << 2)))->source
-						 : (ixs->iph->protocol == IPPROTO_TCP
-						    ? ((struct tcphdr*)((caddr_t)ixs->iph + (ixs->iph->ihl << 2)))->source
-						    : 0));
-					hold_eroute.er_dst_port = 
-						(ixs->iph->protocol == IPPROTO_UDP
-						 ? ((struct udphdr*) (((caddr_t)ixs->iph) + (ixs->iph->ihl << 2)))->dest
-						 : (ixs->iph->protocol == IPPROTO_TCP
-						    ? ((struct tcphdr*)((caddr_t)ixs->iph + (ixs->iph->ihl << 2)))->dest
-						    : 0));
-#endif /* IPSEC_CONFIG_FULL_SELECTOR_LIST */
-						
-#ifdef CONFIG_IPSEC_DEBUG
-					if (debug_pfkey) {
-						char buf1[64], buf2[64];
-						subnettoa(hold_eroute.er_eaddr.sen_ip_src,
-							  hold_eroute.er_emask.sen_ip_src, 0, buf1, sizeof(buf1));
-						subnettoa(hold_eroute.er_eaddr.sen_ip_dst,
-							  hold_eroute.er_emask.sen_ip_dst, 0, buf2, sizeof(buf2));
-						KLIPS_PRINT(debug_pfkey,
-							    "klips_debug:ipsec_xmit_encap_bundle: "
-							    "calling breakeroute and makeroute for %s->%s HOLD eroute.\n",
-							    buf1, buf2);
-					}
-#endif /* CONFIG_IPSEC_DEBUG */
-
-					ixs->error = ipsec_breakroute(&(hold_eroute.er_eaddr),
-							 &(hold_eroute.er_emask),
-							 &first, &last);
-					if (ixs->error != EINVAL) {
-						KLIPS_PRINT(debug_pfkey,
-							    "klips_debug:ipsec_xmit_encap_bundle: "
-							    "breakeroute should have failed.  first=0p%p, last=0p%p\n",
-							    first, last);
-					} else {
-						KLIPS_PRINT(debug_pfkey,
-							    "klips_debug:ipsec_xmit_encap_bundle: "
-							    "HOLD breakeroute found nothing as expected.\n");
-					}
-					if(first != NULL) {
-						ipsec_kfree_skb(first);
-					}
-					if(last != NULL) {
-						ipsec_kfree_skb(last);
-					}
-#if 0
-					if (ixs->error != EINVAL) {
-						SENDERR(-error);
-					}
-#endif
-						
-					if ((ixs->error = ipsec_makeroute(&(hold_eroute.er_eaddr),
-									  &(hold_eroute.er_emask),
-									  hold_said,
-									  ixs->eroute_pid,
-									  ixs->skb,
-									  NULL,
-									  NULL))) {
-						KLIPS_PRINT(debug_pfkey,
-							    "klips_debug:ipsec_xmit_encap_bundle: "
-							    "HOLD makeroute returned %d, failed.\n",
-							    ixs->error);
-						/* SENDERR(-ixs->error); */
-					} else {
-						KLIPS_PRINT(debug_pfkey,
-							    "klips_debug:ipsec_xmit_encap_bundle: "
-							    "HOLD makeroute call successful.\n");
-						ixs->skb = NULL;
-					}
-				}
-			}
-			ixs->stats->tx_dropped++;
-		}
-		default:
-			/* XXX what do we do with an unknown shunt spi? */
-			break;
-		} /* switch (ntohl(ixs->outgoing_said.spi)) */
-		return IPSEC_XMIT_STOLEN;
-	} /* if (ixs->outgoing_said.proto == IPPROTO_INT) */
+		return ipsec_xmit_encap_internal(ixs);
+	}
 		
 	/*
 	  The spinlock is to prevent any other process from
@@ -1195,7 +883,7 @@ ipsec_xmit_encap_bundle(struct ipsec_xmit_state *ixs)
 	spin_lock(&tdb_lock);
 
 	ixs->ipsp = ipsec_sa_getbyid(&ixs->outgoing_said);
-	ixs->sa_len = satoa(ixs->outgoing_said, 0, ixs->sa_txt, SATOA_BUF);
+	ixs->sa_len = satot(&ixs->outgoing_said, 0, ixs->sa_txt, sizeof(ixs->sa_txt));
 
 	if (ixs->ipsp == NULL) {
 		KLIPS_PRINT(debug_tunnel & DB_TN_XMIT,
@@ -1221,7 +909,7 @@ ipsec_xmit_encap_bundle(struct ipsec_xmit_state *ixs)
 	 */
 	ixs->ipsq = ixs->ipsp;	/* save the head of the ipsec_sa chain */
 	while (ixs->ipsp) {
-		ixs->sa_len = satoa(ixs->ipsp->ips_said, 0, ixs->sa_txt, SATOA_BUF);
+		ixs->sa_len = satot(&ixs->ipsp->ips_said, 0, ixs->sa_txt, sizeof(ixs->sa_txt));
 		if(ixs->sa_len == 0) {
 			strcpy(ixs->sa_txt, "(error)");
 		}
@@ -1299,35 +987,30 @@ ipsec_xmit_encap_bundle(struct ipsec_xmit_state *ixs)
 			    IPS_XFORM_NAME(ixs->ipsp),
 			    ixs->sa_len ? ixs->sa_txt : " (error)");
 		switch(ixs->ipsp->ips_said.proto) {
-#ifdef CONFIG_IPSEC_AH
-		case IPPROTO_AH:
-			ixs->headroom += sizeof(struct ahhdr);
-			break;
-#endif /* CONFIG_IPSEC_AH */
-#ifdef CONFIG_IPSEC_ESP
+#ifdef CONFIG_KLIPS_ESP
 		case IPPROTO_ESP:
 			switch(ixs->ipsp->ips_encalg) {
-#ifdef CONFIG_IPSEC_ENC_3DES
+#ifdef CONFIG_KLIPS_ENC_3DES
 			case ESP_3DES:
 				ixs->headroom += sizeof(struct esphdr);
 				break;
-#endif /* CONFIG_IPSEC_ENC_3DES */
+#endif /* CONFIG_KLIPS_ENC_3DES */
 			default:
 				ixs->stats->tx_errors++;
 				bundle_stat = IPSEC_XMIT_ESP_BADALG;
 				goto cleanup;
 			}
 			switch(ixs->ipsp->ips_authalg) {
-#ifdef CONFIG_IPSEC_AUTH_HMAC_MD5
+#ifdef CONFIG_KLIPS_AUTH_HMAC_MD5
 			case AH_MD5:
 				ixs->tailroom += AHHMAC_HASHLEN;
 				break;
-#endif /* CONFIG_IPSEC_AUTH_HMAC_MD5 */
-#ifdef CONFIG_IPSEC_AUTH_HMAC_SHA1
+#endif /* CONFIG_KLIPS_AUTH_HMAC_MD5 */
+#ifdef CONFIG_KLIPS_AUTH_HMAC_SHA1
 			case AH_SHA:
 				ixs->tailroom += AHHMAC_HASHLEN;
 				break;
-#endif /* CONFIG_IPSEC_AUTH_HMAC_SHA1 */
+#endif /* CONFIG_KLIPS_AUTH_HMAC_SHA1 */
 			case AH_NONE:
 				break;
 			default:
@@ -1337,14 +1020,14 @@ ipsec_xmit_encap_bundle(struct ipsec_xmit_state *ixs)
 			}			
 			ixs->tailroom += ((8 - ((ixs->pyldsz + 2 * sizeof(unsigned char)) % 8)) % 8) + 2;
 			break;
-#endif /* !CONFIG_IPSEC_ESP */
-#ifdef CONFIG_IPSEC_IPIP
+#endif /* !CONFIG_KLIPS_ESP */
+#ifdef CONFIG_KLIPS_IPIP
 		case IPPROTO_IPIP:
 			ixs->headroom += sizeof(struct iphdr);
 			break;
-#endif /* !CONFIG_IPSEC_IPIP */
+#endif /* !CONFIG_KLIPS_IPIP */
 		case IPPROTO_COMP:
-#ifdef CONFIG_IPSEC_IPCOMP
+#ifdef CONFIG_KLIPS_IPCOMP
 			/*
 			  We can't predict how much the packet will
 			  shrink without doing the actual compression.
@@ -1359,7 +1042,7 @@ ipsec_xmit_encap_bundle(struct ipsec_xmit_state *ixs)
 			  At least we know the packet will not grow.
 			*/
 			break;
-#endif /* CONFIG_IPSEC_IPCOMP */
+#endif /* CONFIG_KLIPS_IPCOMP */
 		default:
 			ixs->stats->tx_errors++;
 			bundle_stat = IPSEC_XMIT_BADPROTO;
@@ -1573,14 +1256,38 @@ ipsec_xmit_encap_bundle(struct ipsec_xmit_state *ixs)
 	 */
 	while (ixs->ipsp) {
 		enum ipsec_xmit_value encap_stat = IPSEC_XMIT_OK;
+		struct xform_functions *protofuncs;
 
-		encap_stat = ipsec_xmit_encap_once(ixs);
+		switch(ixs->ipsp->ips_said.proto) {
+#ifdef CONFIG_KLIPS_ESP
+		case IPPROTO_ESP:
+		  protofuncs = esp_xform_funcs;
+		  break;
+		break;
+#endif /* !CONFIG_KLIPS_ESP */
+#ifdef CONFIG_KLIPS_IPIP
+		case IPPROTO_IPIP:
+		  protofuncs = ipip_xform_funcs;
+		  break;
+#endif /* !CONFIG_KLIPS_IPIP */
+#ifdef CONFIG_KLIPS_IPCOMP
+		case IPPROTO_COMP:
+		  protofuncs = ipcomp_xform_funcs;
+		  break;
+#endif /* CONFIG_KLIPS_IPCOMP */
+		default:
+		  ixs->stats->tx_errors++;
+		  bundle_stat = IPSEC_XMIT_BADPROTO;
+		  goto cleanup; 
+		}
+
+		encap_stat = ipsec_xmit_encap_once(ixs, protofuncs);
 		if(encap_stat != IPSEC_XMIT_OK) {
 			KLIPS_PRINT(debug_tunnel & DB_TN_XMIT,
 				    "klips_debug:ipsec_xmit_encap_bundle: encap_once failed: %d\n",
 				    encap_stat);
 				
-			bundle_stat = IPSEC_XMIT_ENCAPFAIL;
+			bundle_stat = encap_stat;
 			goto cleanup;
 		}
 	}
@@ -1592,6 +1299,45 @@ ipsec_xmit_encap_bundle(struct ipsec_xmit_state *ixs)
 
 /*
  * $Log: ipsec_xmit.c,v $
+ * Revision 1.14  2004/02/24 17:17:04  mcr
+ * 	s/CONFIG_IPSEC/CONFIG_KLIPS/ as 26sec uses "CONFIG_IPSEC" to
+ * 	turn it on/off as well.
+ *
+ * Revision 1.13  2004/02/16 04:41:19  mcr
+ * 	re-factored IPPROTO_INT code into ipsec_xmit_encap_internal()
+ * 	fixed code - if we are going to send and %trap (therefore
+ * 	save), then we should copy it. Note, we are going to send the
+ * 	packet *TWICE*. A second time through the possible tunnel.
+ * 	maybe this is wrong.
+ * 	more debugging for port-53 examination - pick which which
+ * 	source of port information is non-zero.
+ *
+ * Revision 1.12  2004/02/14 21:59:52  mcr
+ * 	TRAPPASS code - port-53 code.
+ *
+ * Revision 1.11  2004/02/14 21:13:25  mcr
+ * 	added TRAPPASS handling code.
+ *
+ * Revision 1.10  2003/12/13 04:09:21  mcr
+ * 	AH transform removed.
+ *
+ * Revision 1.9  2003/12/11 20:15:04  mcr
+ * 	refactored the xmit code, to move all encapsulation
+ * 	code into protocol functions. Note that all functions
+ * 	are essentially done by a single function, which is probably
+ * 	wrong.
+ * 	the rcv_functions structures are renamed xform_functions.
+ *
+ * Revision 1.8  2003/12/06 21:21:38  mcr
+ * 	split up receive path into per-transform files, for
+ * 	easier later removal.
+ *
+ * Revision 1.7  2003/12/04 19:05:55  mcr
+ * 	cleaned up "sa_id" structure to use "ip_said" only.
+ *
+ * Revision 1.6  2003/11/07 02:58:07  mcr
+ * 	backout of port-selector and X.509 patches
+ *
  * Revision 1.4  2003/06/20 02:28:10  mcr
  * 	misstype of variable name, not detected by module build.
  *
