@@ -14,7 +14,7 @@
  * for more details.
  */
 
-char eroute_c_version[] = "RCSID $Id: eroute.c,v 1.56 2003/09/10 00:01:25 mcr Exp $";
+char eroute_c_version[] = "RCSID $Id: eroute.c,v 1.59 2003/12/05 16:10:37 mcr Exp $";
 
 
 #include <sys/types.h>
@@ -44,7 +44,6 @@ char eroute_c_version[] = "RCSID $Id: eroute.c,v 1.56 2003/09/10 00:01:25 mcr Ex
 
 #include "freeswan/radij.h"
 #include "freeswan/ipsec_encap.h"
-#include "freeswan/ipsec_netlink.h"
 
 #include <stdio.h>
 #include <getopt.h>
@@ -60,6 +59,21 @@ int pfkey_sock;
 fd_set pfkey_socks;
 uint32_t pfkey_seq = 0;
 
+#define EMT_IFADDR	1	/* set enc if addr */
+#define EMT_SETSPI	2	/* Set SPI properties */
+#define EMT_DELSPI	3	/* Delete an SPI */
+#define EMT_GRPSPIS	4	/* Group SPIs (output order)  */
+#define EMT_SETEROUTE	5	/* set an extended route */
+#define EMT_DELEROUTE	6	/* del an extended route */
+#define EMT_TESTROUTE	7	/* try to find route, print to console */
+#define EMT_SETDEBUG	8	/* set debug level if active */
+#define EMT_UNGRPSPIS	9	/* UnGroup SPIs (output order)  */
+#define EMT_CLREROUTE	10	/* clear the extended route table */
+#define EMT_CLRSPIS	11	/* clear the spi table */
+#define EMT_REPLACEROUTE	12	/* set an extended route */
+#define EMT_GETDEBUG	13	/* get debug level if active */
+#define EMT_INEROUTE	14	/* set incoming policy for IPIP on a chain */
+
 static void
 usage(char* arg)
 {
@@ -746,6 +760,13 @@ main(int argc, char **argv)
 }
 /*
  * $Log: eroute.c,v $
+ * Revision 1.59  2003/12/05 16:10:37  mcr
+ * 	patches to avoid ipsec_netlink.h, which has been obsolete for
+ * 	some time now.
+ *
+ * Revision 1.58  2003/11/07 02:58:07  mcr
+ * 	backout of port-selector and X.509 patches
+ *
  * Revision 1.56  2003/09/10 00:01:25  mcr
  * 	fixes for gcc 3.3 from Matthias Bethke <Matthias.Bethke@gmx.net>
  *
