@@ -12,7 +12,7 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
- * RCSID $Id: state.c,v 1.116 2003/08/16 23:04:41 mcr Exp $
+ * RCSID $Id: state.c,v 1.123 2004/01/15 17:42:09 mcr Exp $
  */
 
 #include <stdio.h>
@@ -46,6 +46,8 @@
 #include "sha1.h"
 #include "md5.h"
 #include "crypto.h" /* requires sha1.h and md5.h */
+#include "diag.h"       /* for LOGWIDTH and buildiag() */
+
 
 /*
  * Global variables: had to go somewhere, might as well be this file.
@@ -363,8 +365,6 @@ delete_state(struct state *st)
     pfreeany(st->st_skeyid_a.ptr);
     pfreeany(st->st_skeyid_e.ptr);
     pfreeany(st->st_enc_key.ptr);
-    pfreeany(st->st_ah.our_keymat);
-    pfreeany(st->st_ah.peer_keymat);
     pfreeany(st->st_esp.our_keymat);
     pfreeany(st->st_esp.peer_keymat);
 
@@ -463,8 +463,7 @@ delete_states_by_connection(struct connection *c)
     }
 }
 
-/*
- * walk through the state table, and delete each state whose phase 1 (IKE)
+/* Walk through the state table, and delete each state whose phase 1 (IKE)
  * peer is among those given.
  */
 void
@@ -473,12 +472,6 @@ delete_states_by_peer(ip_address *peer)
     char peerstr[ADDRTOT_BUF];
     int i;
 
-    /* We take two passes so that we delete any ISAKMP SAs last.
-     * This allows Delete Notifications to be sent.
-     * ?? We could probably double the performance by caching any
-     * ISAKMP SA states found in the first pass, avoiding a second.
-     */
-
     addrtot(peer, 0, peerstr, sizeof(peerstr));
 
     /* For each hash chain... */
@@ -491,30 +484,22 @@ delete_states_by_peer(ip_address *peer)
 	{
 	    struct state *this = st;
 	    struct spd_route *sr;
-	    struct connection *thisc = this->st_connection;
-	    
+	    struct connection *c = this->st_connection;
+
 	    st = st->st_hashchain_next;	/* before this is deleted */
 
-	    for(sr = &thisc->spd;
-		sr != NULL;
-		sr = sr->next)
+	    /* ??? Is it not the case that the peer is the same for all spds? */
+	    for (sr = &c->spd; sr != NULL; sr = sr->next)
 	    {
-		if(sameaddr(&sr->that.host_addr, peer))
+		if (sameaddr(&sr->that.host_addr, peer))
 		{
-		    break;
+		    plog("peer %s for connection %s deleting - claimed to have crashed"
+			 , peerstr
+			 , c->name);
+		    delete_states_by_connection(c);
+		    break;	/* can only delete it once */
 		}
 	    }
-
-	    /* if we broke out, then we have something here,	    
-	     * we can only delete it once, so do that.
-	     */
-	    if(sr && sameaddr(&sr->that.host_addr, peer))
-	    {
-		plog("peer %s for connection %s deleting - claimed to have crashed"
-		     , peerstr
-		     , thisc->name);
-		delete_states_by_connection(thisc);
-	    }
 	}
     }
 }
@@ -571,13 +556,19 @@ find_state(const u_char *icookie
     struct state *st = *state_hash(icookie, rcookie, peer);
 
     while (st != (struct state *) NULL)
+    {
 	if (sameaddr(peer, &st->st_connection->spd.that.host_addr)
-	&& memcmp(icookie, st->st_icookie, COOKIE_SIZE) == 0
-	&& memcmp(rcookie, st->st_rcookie, COOKIE_SIZE) == 0
-	&& msgid == st->st_msgid)
-	    break;
-	else
-	    st = st->st_hashchain_next;
+	    && memcmp(icookie, st->st_icookie, COOKIE_SIZE) == 0
+	    && memcmp(rcookie, st->st_rcookie, COOKIE_SIZE) == 0)
+	{
+	    DBG(DBG_CONTROL,
+		DBG_log("peer and cookies match, provided msgid %08x vs %08x",
+			ntohl(msgid), ntohl(st->st_msgid)));
+	    if(msgid == st->st_msgid)
+		break;
+	}
+	st = st->st_hashchain_next;
+    }
 
     DBG(DBG_CONTROL,
 	if (st == NULL)
@@ -610,9 +601,14 @@ find_sender(size_t packet_len, u_char *packet)
     return NULL;
 }
 
+/*
+ * protoid was relevant when AH was present.
+ *
+ */
+
 struct state *
 find_phase2_state_to_delete(const struct state *p1st
-, u_int8_t protoid
+, u_int8_t protoid UNUSED
 , ipsec_spi_t spi
 , bool *bogus)
 {
@@ -628,8 +624,7 @@ find_phase2_state_to_delete(const struct state *p1st
 	    && p1st->st_connection->host_pair == st->st_connection->host_pair
 	    && same_peer_ids(p1st->st_connection, st->st_connection, NULL))
 	    {
-		struct ipsec_proto_info *pr = protoid == PROTO_IPSEC_AH
-		    ? &st->st_ah : &st->st_esp;
+		struct ipsec_proto_info *pr = &st->st_esp;
 
 		if (pr->present)
 		{
@@ -771,11 +766,6 @@ void fmt_state(struct state *st, time_t n
 	}
 
 	*p = '\0';
-	if (st->st_ah.present)
-	{
-	    add_said(&c->spd.that.host_addr, st->st_ah.attrs.spi, SA_AH);
-	    add_said(&c->spd.this.host_addr, st->st_ah.our_spi, SA_AH);
-	}
 	if (st->st_esp.present)
 	{
 	    add_said(&c->spd.that.host_addr, st->st_esp.attrs.spi, SA_ESP);
@@ -787,8 +777,7 @@ void fmt_state(struct state *st, time_t n
 	    add_said(&c->spd.this.host_addr, st->st_ipcomp.our_spi, SA_COMP);
 	}
 #ifdef KLIPS
-	if (st->st_ah.attrs.encapsulation == ENCAPSULATION_MODE_TUNNEL
-	|| st->st_esp.attrs.encapsulation == ENCAPSULATION_MODE_TUNNEL
+	if (st->st_esp.attrs.encapsulation == ENCAPSULATION_MODE_TUNNEL
 	|| st->st_ipcomp.attrs.encapsulation == ENCAPSULATION_MODE_TUNNEL)
 	{
 	    add_said(&c->spd.that.host_addr, st->st_tunnel_out_spi, SA_IPIP);
@@ -833,10 +822,7 @@ show_states_status(void)
 {
     time_t n = now();
     int i;
-    char state_buf[LOG_WIDTH];
-    char state_buf2[LOG_WIDTH];
     int count;
-    struct state **array;
 
     /* make count of states */
     count = 0;
@@ -850,42 +836,50 @@ show_states_status(void)
 	}
     }
 
-    /* build the array */
-    array = alloc_bytes(sizeof(struct state *)*count, "state array");
-    count = 0;
-    for (i = 0; i < STATE_TABLE_SIZE; i++)
+    /* Note: the following code could be executed even if count == 0
+     * but would trigger complaints from Electric Fence about malloc(0).
+     * Besides, this way is more efficient.
+     */
+    if (count > 0)
     {
-	struct state *st;
+	/* build the array */
+	struct state **array = alloc_bytes(sizeof(struct state *)*count, "state array");
 
-	for (st = statetable[i]; st != NULL; st = st->st_hashchain_next)
+	count = 0;
+	for (i = 0; i < STATE_TABLE_SIZE; i++)
 	{
-	    array[count++]=st;
+	    struct state *st;
+
+	    for (st = statetable[i]; st != NULL; st = st->st_hashchain_next)
+	    {
+		array[count++]=st;
+	    }
 	}
+
+	/* sort it! */
+	qsort(array, count, sizeof(struct state *), state_compare);
+
+	/* now print sorted results */
+	for (i = 0; i < count; i++)
+	{
+	    struct state *st = array[i];
+	    char state_buf[LOG_WIDTH];
+	    char state_buf2[LOG_WIDTH];
+
+	    fmt_state(st, n, state_buf, sizeof(state_buf)
+		      , state_buf2, sizeof(state_buf2));
+	    whack_log(RC_COMMENT, state_buf);
+	    if (state_buf2[0] != '\0')
+		whack_log(RC_COMMENT, state_buf2);
+
+	    /* show any associated pending Phase 2s */
+	    if (IS_PHASE1(st->st_state))
+		show_pending_phase2(st->st_connection->host_pair, st);
+	}
+
+	/* free the array */
+	pfree(array);
     }
-
-    /* sort it! */
-    qsort(array, count, sizeof(struct state *), state_compare);
-
-    /* now print sorted results */
-    for (i = 0; i < count; i++)
-    {
-	struct state *st;
-
-	st = array[i];
-
-	fmt_state(st, n, state_buf, sizeof(state_buf)
-		  , state_buf2, sizeof(state_buf2));
-	whack_log(RC_COMMENT, state_buf);
-	if (state_buf2[0] != '\0')
-	    whack_log(RC_COMMENT, state_buf2);
-
-	/* show any associated pending Phase 2s */
-	if (IS_PHASE1(st->st_state))
-	    show_pending_phase2(st->st_connection->host_pair, st);
-    }
-
-    /* free the array */
-    pfree(array);
 }
 
 /* Given that we've used up a range of unused CPI's,
