@@ -9,7 +9,7 @@ named
 # verify that we have some dns data!
 dig sunrise-oe.uml.freeswan.org a
 
-netstat -rn
+netstat -rne
 
 ipsec setup start
 
