@@ -4,7 +4,7 @@ ifconfig eth0 inet 192.1.3.210
 route delete -net default 
 route add -net default gw 192.1.3.254
 
-netstat -rn
+netstat -rne
 
 named
 sleep 2
