@@ -1,7 +1,7 @@
 #! /bin/sh 
 #
 # 
-# $Id: uml-functions.sh,v 1.28 2003/09/02 19:45:48 mcr Exp $
+# $Id: uml-functions.sh,v 1.29 2003/10/31 02:43:34 mcr Exp $
 #
 
 setup_make() {
@@ -17,6 +17,7 @@ setup_make() {
     esac
 
     echo "IPSECDIR=${FREESWANSRCDIR}/linux/net/ipsec"
+    echo "include ${FREESWANSRCDIR}/Makefile.ver"
     echo "module/ipsec.o: ${FREESWANSRCDIR}/packaging/makefiles/module.make \${IPSECDIR}/*.c"
     echo "$TAB mkdir -p module"
     echo "$TAB make -C ${FREESWANSRCDIR} FREESWANSRCDIR=${FREESWANSRCDIR} MODBUILDDIR=$POOLSPACE/module KERNELSRC=$POOLSPACE/plain ARCH=um SUBARCH=${SUBARCH} module "
@@ -274,6 +275,12 @@ setup_host() {
 
 #
 # $Log: uml-functions.sh,v $
+# Revision 1.29  2003/10/31 02:43:34  mcr
+# 	pull up of port-selector tests
+#
+# Revision 1.28.2.1  2003/10/29 02:11:00  mcr
+# 	make sure that local module makefile gets version info included.
+#
 # Revision 1.28  2003/09/02 19:45:48  mcr
 # 	use rootfs= directive instead of ubd0= directive for
 # 	setting hostfs root file system.
